<?php

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class WC_Gateway_Mipa_Base extends WC_Payment_Gateway
{

    const id = 'mipa_base';
    var $debug = FALSE;
    var $payMiPa;
    var $settings = array();

    /**
     * Constructor for the gateway.
     * @access public
     * @return void
     */
    public function __construct()
    {
        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        //mail('te@st.de', 'settings', print_r($this->settings, 1));
        //mail('te@st.de', 'form_fields', print_r($this->form_fields, 1));

        // Define user set variables after init_settings
        foreach ($this->form_fields AS $k => $v) {
            $this->$k = $this->get_option($k);
        }
        //mail('te@st.de', 'this', print_r($this, 1));

        $lang = $this->getLanguage();
        if (empty($lang)) $lang = 'de';
        if (!empty($this->get_option('title_'.$lang))){
            $this->title = $this->get_option('title_'.$lang);
        }
        $this->description = $this->get_option('description_'.$lang);

        require_once($this->plugin_path() . 'mcpay/classes/class.mcpay.php');
        $this->payMiPa = new mcpay();

        $this->icon = $this->payMiPa->getLogo(str_replace('mipa_','',$this->id), NULL, false);
//        $this->icon = 'none';
//        $this->title = $this->payMiPa->getLogo(str_replace('mipa_','',$this->id), NULL, true).$this->title;

        // Logs
        if ($this->debug) {
            $this->log = new WC_Logger();
            $this->payMiPa->log = new WC_Logger();
            $this->payMiPa->debug = $this->debug;
        }

        // Actions
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options')); // save admin options
        add_action('woocommerce_check_cart_items', array($this, 'check_cart_items')); // fix display error on checkout page

        $this->supports = array(
            'products',
            'refunds',
            'tokenization',
        );
        // Subscriptions
        if (class_exists('WC_Subscriptions_Order')) {
            // check if subscription is supported
            if (in_array(str_replace('mipa_', '', $this->id), $this->payMiPa->subscriptionPaymethods)) {
                $this->supports = array(
                    'products',
                    'refunds',
                    'tokenization',
                    'subscriptions',
                    'subscription_cancellation',
                    'subscription_suspension',
                    'subscription_reactivation',
                    'subscription_amount_changes',
                    'subscription_date_changes',
                    'subscription_payment_method_change',
                );

                add_action('woocommerce_scheduled_subscription_payment_' . $this->id, array($this, 'scheduled_subscription_payment'), 10, 2);
            }
        }

        if (!$this->is_valid_for_use()) {
            $this->enabled = FALSE;
        }
    }

    /**
     * we need to set something into $_POST
     * because checkout page would redirect to shopping cart
     * if post is empty and wc_add_notice with type error is set.
     * see thanksAction
     */
    public function check_cart_items()
    {
        $_POST['mipa_show_error'] = 1;
    }

    /**
     * get_icon
     *
     * @return mixed|string|void
     */
    public function get_icon()
    {
        if (strtolower($this->icon) == 'none') return false;
        $icons = '<img src="' . $this->plugin_url() . 'images/' . $this->id . '.png' . '" xstyle="max-height: 25px; padding: 0 5px; display: inline-block"/>';
        if (!empty($this->icon)) {
            $icons = '<img src="' . $this->icon . '" xstyle="max-height: 30px; padding: 0 5px 0 0; display: inline-block; float: right; margin: 0;"/>';
        }
        return apply_filters('woocommerce_gateway_icon', $icons, $this->id);
    }

    /**
     * get_poweredby
     *
     * @return string|void
     */
    public function get_poweredby()
    {
        return __('<div style="font-size: 10px;float: right;color: #bbb">powered by <a href="http://www.micropayment.de" title="Payment Service Provider" target="_blank" style="color: #000">micro<span style="color: #800">payment</span></a></div>', 'mipa');
    }

    /**
     * Check if this gateway is enabled and available in the user's country
     * @access public
     * @return bool
     */
    public function is_valid_for_use()
    {
        global $woocommerce;

        //mail('te@st.de', __CLASS__.'->'.__FUNCTION__.'->settings', print_r($this->settings,1));
//        if (empty($woocommerce->session->mipa_allowed_currenies)) {
//            @$woocommerce->session->mipa_allowed_currenies = $this->payMiPa->getAllowedCurrenies($this->settings);
//        }
//        $allowedCurrency = $woocommerce->session->mipa_allowed_currenies;
        $allowedCurrency = $this->payMiPa->getAllowedCurrenies($this->settings);
        //mail('te@st.de', __CLASS__.'->'.__FUNCTION__.'->allowedCurrency', print_r($allowedCurrency,1));

        $actCurrency = strtoupper(get_woocommerce_currency());
        if (!in_array($actCurrency, $allowedCurrency)) {
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->currency_not_allowed: ' . get_woocommerce_currency() . ' not in ' . print_r($allowedCurrency, 1) . "\n" . print_r($woocommerce->cart, 1));
            return FALSE;
        }

        if (!empty($woocommerce->cart->total)) {
            $actAmount = $woocommerce->cart->total * 100;
            if ($actAmount < $this->mini_amount) {
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->min_amount: ' . $actAmount . ' < ' . $this->mini_amount . "\n" . print_r($woocommerce->cart, 1));
                return FALSE;
            }
            if ($this->maxi_amount > 0 && $actAmount > $this->maxi_amount) {
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->maxi_amount: ' . $actAmount . '> ' . $this->maxi_amount . "\n" . print_r($woocommerce->cart, 1));
                return FALSE;
            }
        }

        // check virtual products
        $onlyVirtualAllowed = array('mipa_call2pay', 'mipa_handypay');
        if (in_array($this->id, $onlyVirtualAllowed) && !$this->cartIsVirtual()) return FALSE;

        return TRUE;
    }

    /**
     * Check if paymethod is activated
     * @access public
     * @return bool
     */
    public function is_active_payment($code)
    {
        return $this->get_option('activate_' . $code) == 'yes';
    }

    /**
     * admin_options
     * Admin Panel Options
     * - Options for bits like 'title' and availability on a country-by-country basis
     * @since 1.0.0
     */
    public function admin_options()
    {
        echo '<h3>' . __('Micropayment', 'mipa') . '</h3>';
        if ($this->is_valid_for_use()) {
            echo '<table class="form-table">';
            $this->generate_settings_html();
            echo '</table>';
        } else {
            echo '<div class="inline error"><p>
                    <strong>' . __('Gateway Disabled', 'mipa') . '</strong>:'
                . __('Micropayment does not support your store currency.', 'mipa') . '
                  </p></div>';
        }
    }

    /**
     * init_form_fields
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array();
        $baseFields = $this->get_base_form_fields();
        $this->form_fields = array_merge($this->form_fields, $baseFields);
    }

    /**
     * get_base_form_fields
     * Initialise Gateway Settings Form Fields
     * @access public
     * @return void
     */
    public function get_base_form_fields()
    {
        $possibleStatus = wc_get_order_statuses();
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$possibleStatus', print_r($possibleStatus,1));

        return array(
            'enabled' => array(
                'title' => __('Enable/Disable', 'mipa'),
                'type' => 'checkbox',
                'label' => __('Enable Plugin', 'mipa'),
                'default' => 'yes'
            ),
            'title_de' => array(
                'title' => __('Title', 'mipa').' DE',
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'mipa'),
                'default' => '',
                'desc_tip' => TRUE,
            ),
            'title_en' => array(
                'title' => __('Title', 'mipa').' EN',
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'mipa'),
                'default' => '',
                'desc_tip' => TRUE,
            ),
            'description_de' => array(
                'title' => __('Description', 'mipa').' DE',
                'type' => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'mipa'),
                'default' => ''
            ),
            'description_en' => array(
                'title' => __('Description', 'mipa').' EN',
                'type' => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'mipa'),
                'default' => ''
            ),

            'credentials' => array(
                'title' => __('Credentials', 'mipa'),
                'type' => 'title',
                'description' => '',
            ),
            'accessKey' => array(
                'title' => __('AccessKey', 'mipa'),
                'description' => __('This can be found under <strong>Configuration &raquo; Accesskey</strong> in your micropayment account.', 'mipa'),
                'type' => 'text',
                'default' => '', // 9e7774d41a312c464cd4b8b36b7954a2
                'desc_tip' => TRUE,
                'placeholder' => 'abcdef0123456789abcdef0123456789'
            ),
            'project' => array(
                'title' => __('Project Shortcut', 'mipa'),
                'description' => __('This can be found under <strong>Configuration &raquo; Projects</strong>. Call a project and copy the shortcut.', 'mipa'),
                'type' => 'text',
                'default' => '', // 11cu-jovet-0df2fd78
                'desc_tip' => TRUE,
                'placeholder' => 'abcd-ef0123-456789'
            ),
            'testMode' => array(
                'title' => __('Test Mode', 'mipa'),
                'description' => __('Here you can switch between Live and Test Mode.', 'mipa'),
                'type' => 'select',
                'default' => 'true',
                'desc_tip' => TRUE,
                'options' => array(
                    1 => __('Test', 'mipa'),
                    0 => __('Live', 'mipa')
                )
            ),
            'debug' => array(
                'title' => __('Debug Log Activation', 'mipa'),
                'description' => __('Activate Debug Logging.', 'mipa'),
                'type' => 'select',
                'default' => 'true',
                'desc_tip' => TRUE,
                'options' => array(
                    TRUE => __('YES', 'mipa'),
                    FALSE => __('NO', 'mipa')
                )
            ),

            'settings' => array(
                'title' => __('Settings', 'mipa'),
                'type' => 'title',
                'description' => '',
            ),
            'freetext' => array(),
            'error_page' => array(
                'title' => __('Error Page', 'mipa'),
                'description' => __('Choose to which page the customer should jump in case of error.', 'mipa'),
                'type' => 'select',
                'default' => 'CHECKOUT',
                'desc_tip' => TRUE,
                'options' => array(
                    'CHECKOUT' => __('Checkout', 'mipa'),
                    'CART' => __('Shopping Cart', 'mipa')
                )
            ),
            'status_paid' => array(
                'title' => __('Status Paid', 'mipa'),
                'description' => __('Choose the status to be set in case of payment success.', 'mipa'),
                'type' => 'select',
                'default' => 'wc-processing',
                'desc_tip' => TRUE,
                'options' => $possibleStatus
            ),
            'status_pending' => array(
                'title' => __('Status Pending', 'mipa'),
                'description' => __('Choose the status to be set in case of payment pending.', 'mipa'),
                'type' => 'select',
                'default' => 'wc-on-hold',
                'desc_tip' => TRUE,
                'options' => $possibleStatus
            ),
            'mini_amount' => array(
                'title' => __('Min Amount', 'mipa'),
                'description' => __('Setup the minimum amount for this paymethod. In Euro CENT', 'mipa'),
                'type' => 'text',
                'default' => '0',
                'desc_tip' => TRUE,
            ),
            'maxi_amount' => array(
                'title' => __('Max Amount', 'mipa'),
                'description' => __('Setup the maximum amount for this paymethod. 0 means no Limit! In Euro CENT. In case you need a higher limit, please contact us.', 'mipa'),
                'type' => 'text',
                'default' => '0',
                'desc_tip' => TRUE,
            ),
            'customPrefix' => array(
                'title' => __('Prefix', 'mipa'),
                'type' => 'text',
                'description' => __('Prefix for customer ids, in case of multiple projects over one account.', 'mipa'),
                'default' => 'NONE',
                'desc_tip' => TRUE,
            ),
        );
    }

    /**
     * responseBaseAction
     *
     * @throws \Exception
     */
    function responseBaseAction()
    {
        global $woocommerce;
        $crlf = "\n";
        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->GET: ' . print_r($_GET, 1));
        $return = 'status=ok' . $crlf;

        if (!empty($_GET['orderid'])) {
            $order_id = (int)$_GET['orderid'];
            $order = new WC_Order($order_id);
            //mail('test@test.test', 'DEBUG '.__CLASS__.'->'.__FUNCTION__, print_r($order, 1));
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->order: ' . print_r($order, 1));
        } else {
            $return = 'status=error' . $crlf . 'message=OrderID missing.' . $crlf;
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->return: ' . print_r($return, 1));
            echo $return;
            exit();
        }

        // save trx id if available
        if (!empty($_GET['transactionid'])) {
            $order->set_transaction_id($_GET['transactionid']);
            $order->save();
        } else if (!empty($_GET['transactionId']) && $_GET['transactionId'] != '__transactionId__') {
            $order->set_transaction_id($_GET['transactionId']);
            $order->save();
        }

        $func = '';
        if (!empty($_GET['function'])) {
            $func = strtolower($_GET['function']);
        } else {
            $return = 'status=error' . $crlf . 'message=Function missing.' . $crlf;
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->OID(' . $order_id . ')return: ' . print_r($return, 1));
            echo $return;
            exit();
        }

        // check the secretfield here
        if (!empty($_GET['secretfield']) && $func != 'custinfo') {
            $config = $this->settings;
            if (!empty($config['secretfield']) && $_GET['secretfield'] != $config['secretfield']) {
                $return = 'status=error' . $crlf . 'message=Security problem.' . $crlf;
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->OID(' . $order_id . ')return: ' . print_r($return, 1));
                echo $return;
                exit();
            }
        }

        switch ($func) {
            // debug info
            case 'custinfo':
                // ?wc-api=wc_gateway_mipa_ccard&function=custinfo&ak=ACCESSKEY&orderid=ORDERID
                $config = $this->settings;
                if (!empty($_GET['ak']) && $_GET['ak'] == $config['accessKey']) {
                    echo '<pre>' . print_r($config, 1) . '</pre>';
                    $order = new WC_Order($order_id);
                    echo '<pre>' . print_r($order, 1) . '</pre>';
                    $userId = $order->get_user_id();
                    $tokens = $this->get_tokens($userId);
                    echo '<pre>' . print_r($tokens, 1) . '</pre>';
                    $info = $this->loadOrderParams($order_id);
                    echo '<pre>' . print_r($info, 1) . '</pre>';
                } else {
                    echo 'Not found!';
                }
                exit();
                break;

            case 'billing':
                if ($order->status !== 'completed') {

                    if ($order->get_total() != ($_GET['amount'] / 100)) {
                        $return = 'status=error' . $crlf . 'message=Amount mismatch.' . $crlf;
                        if ($this->debug) $this->log->error(__CLASS__ . '->' . __FUNCTION__ . '->error: Order(' . $order_id . ') Amount mismatch ' . $order->get_total() . ' != ' . ($_GET['amount'] / 100));

                    } else if ($order->get_currency() != $_GET['currency']) {
                        $return = 'status=error' . $crlf . 'message=Currency mismatch.' . $crlf;
                        if ($this->debug) $this->log->error(__CLASS__ . '->' . __FUNCTION__ . '->error: Order(' . $order_id . ') Currency mismatch ' . $order->get_currency() . ' != ' . $_GET['currency']);

                        /*
                        } else if ((int)$order->get_customer_id() != (int)$_GET['customerId']) {
                          $return = 'status=error' . $crlf . 'message=Customer mismatch.' . $crlf;
                          if ($this->debug) $this->log->error(__CLASS__.'->'.__FUNCTION__ . '->error: Order(' . $order_id . ') Customer mismatch ' . $order->get_customer_id() . ' != ' . $_GET['customerId']);
                        */
                    } else {
                        if (!empty($_GET['testmode'])) {
                            $order->add_order_note(__('!! TEST MODE !!', 'mipa'));
                        }
                        $order->add_order_note(__('AUTH:', 'mipa') . $_GET['auth']);
                        $order->payment_complete(); // processing or completed (if virtual + downloadable)
                        $config = $this->settings;
                        if (!$this->isOrderStatus($order->get_status(), $config['status_paid'])) {
                            $order->set_status($config['status_paid']);
                            $order->save();
                        }

                        $return = 'status=ok' . $crlf // ok  | error
                            . 'url=' . $this->get_return_url($order) . $crlf
                            . 'target=_top' . $crlf
                            . 'forward=1' . $crlf
                            . 'message=OK' . $crlf;
                    }
                }
                break;
            case 'storno': // chargeback
                if (!empty($_GET['paypal_reversalreason'])) { // paypal special
                    $order->update_status('cancelled', __('Payment Cancelled. (STORNO)', 'mipa') . ' ' . $_GET['paypal_reversalreason']);
                } else {
                    $order->update_status('cancelled', __('Payment Cancelled. (STORNO)', 'mipa'));
                }
                break;
            case 'backpay': // chargeback reversal
                $order->update_status('refunded', __('Payment Cancellation refunded (BACKPAY)', 'mipa'));
                break;
            case 'refund': // refund
                // partial refund
                if (($order->get_total() * 100) != $_GET['amount']) {
                    $amount = ($_GET['amount'] / 100);
                    $makeRefund = true;
                    if (!empty($_GET['paystatus']) && $_GET['paystatus'] != 'CLOSED'){
                        $makeRefund = false;
                    } else {
                        $makeRefund = true;
                    }
                    if ($makeRefund) {
                        $refund = wc_create_refund(
                            array(
                                'amount' => $amount,
                                'reason' => __('Payment Refund', 'mipa'),
                                'order_id' => $order_id
                            )
                        );
                        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->refund: ' . print_r($refund, 1));
                        $order->add_order_note(__('Partial Refund ', 'mipa') . $amount . ' ' . $_GET['currency']);
                    }
                } else {
                    // full refund
                    $order->update_status('refunded', __('Payment Refund (REFUND)', 'mipa'));
                }
                break;
            case 'quit': // reversal SEPA only
                $order->update_status('cancelled', __('Payment Cancelled. (QUIT)', 'mipa'));
                break;
            default:
                $return = $this->responseAction($func, $order);
                break;
        }
        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->return: ' . print_r($return, 1));
        echo $return;
        $this->payMiPa->markFinished($order->get_order_key(), $func);
        exit();
    }

    /**
     * plugin_url
     * returns the url of the plugin
     * @return string
     */
    function plugin_url()
    {
        return get_option( 'siteurl' );

        if (isset($this->plugin_url)) return $this->plugin_url;

        $actDir = stripslashes(str_replace(dirname($this->plugin_path()), '', $this->plugin_path()));
        if (is_ssl()) {
            return $this->plugin_url = str_replace('http://', 'https://', WP_PLUGIN_URL) . '/' . $actDir;
        } else {
            return $this->plugin_url = WP_PLUGIN_URL . '/' . $actDir;
        }
    }

    /**
     * plugin_path
     * returns the path of the plugin
     * @return string
     */
    function plugin_path()
    {
        return dirname(__FILE__) . '/';
    }

    /**
     * getSessID
     * get SessID from Order
     *
     * @param $orderID
     *
     * @return bool|string
     */
    function getSessID($orderID)
    {
        $sessID = $this->getOrderVal($orderID, 'sessionId');
        if (empty($sessID)){
            $sessID = $this->getOrderNotes($orderID, 'SESSIONID:');
        }
        return $sessID;
    }

    /**
     * getAuthID
     * get AuthID from Order
     *
     * @param $orderID
     *
     * @return bool|string
     */
    function getAuthID($orderID)
    {
        return $this->getOrderVal($orderID, 'transactionAuth');
    }

    /**
     * getTrxID
     * get TrxID from Order
     *
     * @param $orderID
     *
     * @return bool|string
     */
    function getTrxID($orderID)
    {
        $trxID = $this->getOrderVal($orderID, 'transactionId');
        if (empty($trxID)){
            $trxID = $this->getOrderVal($orderID, 'transaction_id');
        }
        // sepa comment
        if (empty($trxID)){
            $trxID = $this->getOrderNotes($orderID, 'PAYTOKEN:');
        }
        // ccard comment
        if (empty($trxID)){
            $trxID = $this->getOrderNotes($orderID, 'TRXID:');
        }
        return $trxID;
    }

    /**
     * getOrderVal
     *
     * @param $orderID
     * @param $keyword
     *
     * @return mixed
     */
    private function getOrderVal($orderID, $keyword)
    {
        $params = $this->loadOrderParams($orderID);
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$params', print_r($params,1));
        return $params->$keyword;
    }

    /**
     * getOrderNotes
     *
     * @param $orderID
     * @param $keyword
     * @return bool|string|string[]
     */
    private function getOrderNotes($orderID, $keyword)
    {
        $val = false;
        $comments = wc_get_order_notes(array('order_id' => $orderID));
        foreach($comments AS $k => $v){
            if (strpos($v->content, $keyword) !== false){
                $val = str_replace($keyword, '', $v->content);
            }
        }
        return $val;
    }

    /**
     * saveOrderParams
     *
     * @param int $order_id
     * @param array $params
     * @param string $key
     */
    function saveOrderParams($order_id, $params, $key = '_mipa_order_params')
    {
        $order = new WC_Order($order_id);
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$order', print_r($order,1));
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$params', print_r($params,1));
        $oldParams = $this->loadOrderParams($order_id);
        if (!empty($oldParams)) {
            $order->update_meta_data($key, $params);
        } else {
            $order->add_meta_data($key, $params);
        }
        $order->save_meta_data();
    }

    /**
     * loadOrderParams
     *
     * @param int $order_id
     * @param string $key
     *
     * @return bool|mixed
     */
    function loadOrderParams($order_id, $key = '_mipa_order_params')
    {
        $order = new WC_Order($order_id);
        $params = $order->get_meta($key);
        if (empty($params)) return FALSE;
        return $params;
    }

    /**
     * isOrderStatus
     *
     * @param $actual
     * @param $shouldBe
     *
     * @return bool
     */
    function isOrderStatus($actual, $shouldBe)
    {
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'', 'wc-'.$actual.' === '.$shouldBe);
        return 'wc-' . $actual === $shouldBe;
    }

    /**
     * get_tokens
     *
     * overridden method because we need token of given user
     * not of logged in user
     *
     * @param $userId
     *
     * @return array
     */
    function get_tokens($userId = NULL)
    {
        if (empty($userId)) return array();
        if ($this->supports('tokenization')) {
            $this->tokens = WC_Payment_Tokens::get_customer_tokens($userId, $this->id);
            //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$this->get_tokens('.$userId.')', print_r($this->tokens,1));
        }
        return $this->tokens;
    }

    /**
     * cartIsVirtual
     *
     * check if cart is totally virtual or not
     *
     * @return bool
     */
    function cartIsVirtual()
    {
        if (empty(WC()->cart)) return TRUE;
        $isVirtual = TRUE;
        $content = WC()->cart->get_cart_contents();
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$content', print_r($content,1));
        foreach ($content AS $k => $v) {
            /*
            $product = wc_get_product($v['product_id']);
            $pData = $product->get_data();
            //echo '<pre>'.print_r($pData,1).'</pre>';
            if ($product->is_virtual('yes')) {
              // is virtual product
            } else if ($pData['virtual'] > 0) {
              // is virtual product variant*/
            //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$v->data', print_r($v['data'],1));
            if ($v['data']->is_virtual('yes')) {
                // is virtual product
            } else {
                $isVirtual = FALSE;
            }
        }
        return $isVirtual;
    }

    /**
     * getUserDataFromOrder
     *
     * @param $order WC_Order
     *
     * @return array
     * @throws \Exception
     */
    function getUserDataFromOrder($order, $config)
    {
        $this->payMiPa->setConfig($config);
        $order_id = $order->get_id();
        $custID = $order->get_customer_id();
        if (empty($custID)) $custID = 'guest-' . $order->get_id(); // guest order
        $custID = $this->payMiPa->unifyID($custID);

        $amount = ($order->get_total() * 100);
        $currency = $order->get_currency();
        $countryISO = $order->get_billing_country();

        // plugin info
        include_once(dirname(__FILE__) . '/../../../wp-admin/includes/plugin.php');
        $pInfo = get_plugin_data(dirname(__FILE__) . '/gateway-mipa.php');
        $pInfo2 = get_plugin_data(dirname(__FILE__) . '/../woocommerce/woocommerce.php');
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$pInfo2', print_r($pInfo2,1));

        $chksumData = array(
            'amount'        => $amount,
            'currency'      => $currency,
            'iso'           => $countryISO,
            'host'          => $_SERVER['HTTP_HOST'],
            'project'       => $config['project'],
            'shoptype'      => 'woocommerce',
            'shopversion'   => $pInfo2['Version'],
            'plugintype'    => 'API',
            'pluginversion' => $pInfo['Version'],
            'refid'         => $order->get_order_key(), //session_id(),
        );

        $userData = array(
            'company' => $order->get_billing_company(),
            'firstName' => $order->get_billing_first_name(),
            'lastName' => $order->get_billing_last_name(),
            'salutation' => 'MR',
            'street' => $order->get_billing_address_1() . ' ' . $order->get_billing_address_2(),
            'zip' => $order->get_billing_postcode(),
            'city' => $order->get_billing_city(),
            'country' => $countryISO,
            'email' => $order->get_billing_email(),
            'ip' => $order->get_customer_ip_address(),
            'id' => $custID,
            'order_id' => $order_id,
            'lang' => $this->getLanguage(),
            'amount' => $amount,
            'currency' => $currency,
            'chksum' => $chksumData, // request security
            'doFinish' => 1, // wordpress needs finish as direct call
        );
        if (!in_array($userData['lang'], array('de', 'en'))) $userData['lang'] = 'en'; // default de
        if (empty($userData['ip'])) $userData['ip'] = $_SERVER['REMOTE_ADDR']; // Falls IP Leer, dann aus dem Server holen
        if (empty($userData['ip'])) $userData['ip'] = $_SERVER['SERVER_ADDR']; // Falls IP Leer, dann aus dem Server holen
        if (empty($userData['country'])) $userData['country'] = 'DE'; // default DE
        // check if customer name is filled
        if (empty($userData['firstName'])) {
            $customerObj = new WC_Customer($order->get_customer_id());
            //echo '<pre>'.print_r($customerObj,1).'</pre>';
            $userData['firstName'] = $customerObj->get_first_name();
            $userData['lastName'] = $customerObj->get_last_name();
        }
        return $userData;
    }

    /**
     * getLanguage
     *
     * @return string
     */
    function getLanguage()
    {
        return strtolower(substr(get_locale(), 0, 2));
    }

}

?>