<?php

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class WC_Gateway_Mipa_CC extends WC_Gateway_Mipa_Base
{

    const id = 'mipa_ccard';

    /**
     * Constructor for the gateway.
     * @access public
     * @return void
     */
    public function __construct()
    {
        $this->id = self::id;
        $this->method_title = __('Micropayment CreditCard', 'mipa');
        $this->has_fields = TRUE;

        // Load the settings
        $this->init_form_fields();
        $this->init_settings();

        // Actions
        add_action('woocommerce_api_wc_gateway_' . $this->id, array($this, 'responseAction'));
        add_action('woocommerce_api_wc_threedsecure_' . $this->id, array($this, 'threedsecureAction'));
        add_action('woocommerce_api_wc_nothreedsecure_' . $this->id, array($this, 'nothreedsecureAction'));
        add_action('woocommerce_api_wc_getcss_' . $this->id, array($this, 'getCSSAction'));
        add_action('woocommerce_api_wc_getjs_' . $this->id, array($this, 'getJSAction'));

        parent::__construct();
    }

    public function getCSSAction()
    {
        global $woocommerce;
        if (empty($_GET['sn'])) die();
        $sn = $_GET['sn'];
        $content = $this->payMiPa->getCSSFile($sn, true);
        echo $content;
        exit();
    }

    public function getJSAction()
    {
        global $woocommerce;
        if (empty($_GET['sn'])) die();
        $sn = $_GET['sn'];
        $content = $this->payMiPa->getJSFile($sn, true);
        echo $content;
        exit();
    }

    /**
     * Initialise Gateway Settings Form Fields
     * @access public
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable/Disable', 'mipa'),
                'type' => 'checkbox',
                'label' => __('Enable Plugin', 'mipa'),
                'default' => 'yes'
            ),
            'title_de' => array(
                'title' => __('Title', 'mipa') . ' DE',
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'mipa'),
                'default' => 'Micropayment Kreditkarte',
                'desc_tip' => TRUE,
            ),
            'title_en' => array(
                'title' => __('Title', 'mipa') . ' EN',
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'mipa'),
                'default' => 'Micropayment CreditCard',
                'desc_tip' => TRUE,
            ),
            'description_de' => array(
                'title' => __('Description', 'mipa') . ' DE',
                'type' => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'mipa'),
                'default' => $this->get_poweredby() . '<b>Bezahlung per Kreditkarte</b><br>Mit diesem einfachen und sicheren Bezahlsystem können Sie direkt mit Ihren Kreditkartendaten bezahlen.<br><br>'
            ),
            'description_en' => array(
                'title' => __('Description', 'mipa') . ' EN',
                'type' => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'mipa'),
                'default' => $this->get_poweredby() . '<b>Payment with creditcard</b><br>With this simple and secure payment system you can pay directly with your credit card details.<br><br>'
            ),
            'mini_amount' => array(
                'title' => __('Min Amount', 'mipa'),
                'description' => __('Setup the minimum amount for this paymethod. In Euro CENT', 'mipa'),
                'type' => 'text',
                'default' => '250',
                'desc_tip' => TRUE,
            ),
            'maxi_amount' => array(
                'title' => __('Max Amount', 'mipa'),
                'description' => __('Setup the maximum amount for this paymethod. 0 means no Limit! In Euro CENT. In case you need a higher limit, please contact us.', 'mipa'),
                'type' => 'text',
                'default' => '150000',
                'desc_tip' => TRUE,
            ),
            'freetext' => array(
                'title' => __('Freetext on settlement', 'mipa'),
                'description' => __('Here you can define a freetext to be shown in your control center.', 'mipa').' '.__('Placeholder {$orderid} = OrderID', 'mipa'),
                'type' => 'text',
                'placeholder' => __('Order {$orderid} Your Freetext!!', 'mipa'),
                'default' => '',
                'desc_tip' => __('If empty only order id will be used.', 'mipa'),
            ),
            'autogenSession' => array(
                'title' => __('Automatic Session Generation', 'mipa'),
                'description' => __('Choose YES if you want to generate the session automatically.', 'mipa'),
                'type' => 'select',
                'default' => 'true',
                'desc_tip' => __('If you choose NO the session id is a combination of projectid and orderid. This is important if you used subscriptions before plugin version 1.7.0.', 'mipa'),
                'options' => array(
                    1 => __('YES', 'mipa'),
                    0 => __('NO', 'mipa')
                )
            ),
            /*
            'showMonthNames' => array(
              'title' => __('Show Month Names', 'mipa'),
              'description' => __('Choose YES if you want the names of month to be shown in drop down.', 'mipa'),
              'type' => 'select',
              'default' => 'FALSE',
              'desc_tip' => TRUE,
              'options' => array(
                TRUE => __('YES', 'mipa'),
                FALSE => __('NO', 'mipa')
              )
            ),
            'preSelectDate' => array(
              'title' => __('Pre Select Date', 'mipa'),
              'description' => __('Choose YES if you want to pre select the actual month as expire date.', 'mipa'),
              'type' => 'select',
              'default' => 'FALSE',
              'desc_tip' => TRUE,
              'options' => array(
                TRUE => __('YES', 'mipa'),
                FALSE => __('NO', 'mipa')
              )
            ),
            */
            'doublet' => array(
                'title' => __('Allow double', 'mipa'),
                'description' => __('Choose YES if you want to allow that multiple customers can have the same creditcard.', 'mipa'),
                'type' => 'select',
                'default' => 'true',
                'desc_tip' => TRUE,
                'options' => array(
                    1 => __('YES', 'mipa'),
                    0 => __('NO', 'mipa')
                )
            ),
            'sendMail' => array(
                'title' => __('Payment EMails', 'mipa'),
                'description' => __('Choose YES if you want emails sent from Micropayment Server to your customer.', 'mipa'),
                'type' => 'select',
                'default' => 'false',
                'desc_tip' => TRUE,
                'options' => array(
                    TRUE => __('YES', 'mipa'),
                    FALSE => __('NO', 'mipa')
                )
            ),
            'error_color' => array(
                'title' => __('Field Color Error', 'mipa'),
                'description' => __('Choose the color of input fields in case of error.', 'mipa'),
                'type' => 'text',
                'default' => '#ffffaa',
                'desc_tip' => TRUE,
                'placeholder' => '#ffffaa'
            ),
            'error_off_color' => array(
                'title' => __('Field Color no Error', 'mipa'),
                'description' => __('Choose the color of input fields in case of no error.', 'mipa'),
                'type' => 'text',
                'default' => '#ffffff',
                'desc_tip' => TRUE,
                'placeholder' => '#ffffff'
            ),
            'futureYears' => array(
                'title' => __('Show Future Years', 'mipa'),
                'description' => __('Choose how many years in future should be selectable.', 'mipa'),
                'type' => 'text',
                'default' => '10',
                'desc_tip' => TRUE,
                'placeholder' => '10'
            ),
            /*
            'start3D' => array(
                'title' => __('Use 3D Secure', 'mipa'),
                'description' => __('Choose YES if you want to use 3D Secure.', 'mipa'),
                'type' => 'select',
                'default' => 'true',
                'desc_tip' => __('Please contact our support team to check if you can deactivate 3D Secure!', 'mipa'),
                'options' => array(
                    1 => __('YES', 'mipa'),
                    0 => __('NO', 'mipa')
                )
            ),
            'start3Dreuse' => array(
                'title' => __('Use 3D Secure on saved cards', 'mipa'),
                'description' => __('Choose YES if you want to use 3D Secure even with saved cards.', 'mipa'),
                'type' => 'select',
                'default' => 'true',
                'desc_tip' => __('Please contact our support team to check if you can deactivate 3D Secure!', 'mipa'),
                'options' => array(
                    1 => __('YES', 'mipa'),
                    0 => __('NO', 'mipa')
                )
            ),
            */
        );
        $baseFields = $this->get_base_form_fields();
        unset($baseFields['status_pending']); // prepay only
        $this->form_fields = array_merge($baseFields, $this->form_fields);
    }

    /**
     * payment_fields
     * @return bool
     */
    function payment_fields()
    {
        global $woocommerce;
        $config = $this->settings;
        $config['plugin_path'] = $this->plugin_path();
        $config['plugin_url'] = $this->plugin_url();
//        $config['plugin_urljs'] = $this->plugin_url() . 'mcpay/js/';
//        $config['plugin_urlcss'] = $this->plugin_url() . 'mcpay/css/';
        $config['plugin_urljs'] = '?wc-api=wc_getjs_mipa_ccard&sn=';
        $config['plugin_urlcss'] = '?wc-api=wc_getcss_mipa_ccard&sn=';

        $onCheckoutPage = TRUE;
        if (!is_checkout()) $onCheckoutPage = FALSE;
        if (is_checkout_pay_page()) $onCheckoutPage = FALSE;

        $woocommerce->session->onCheckoutPage = $onCheckoutPage; // remember which startpage

        $addPaymethodPage = TRUE;
        if (!is_add_payment_method_page()) $addPaymethodPage = FALSE;

        $this->payMiPa->setConfig($config);
        $mipaJSInline = $this->payMiPa->getStartJS(TRUE);

        $filename = $config['plugin_path'] . 'mcpay/view/mcpay_form_ccard.tpl';
        $showRemember = TRUE;
        $prefillData = array();
        $customerID = $woocommerce->customer->get_id();
        if (empty($customerID)) {
            // guest or new customer
            $showRemember = FALSE;
        } else {
            // load saved paydata old
            $tokens = $this->get_tokens($customerID);
            foreach ($tokens AS $k => $token) {
                $prefillData[$k] = array(
                    'filldesc' => $token->get_display_name(),
                    'token' => $token->get_token(),
                    'holder' => $token->get_card_type(),
                    'pan' => $token->get_last4(),
                    'month' => $token->get_expiry_month(),
                    'year' => $token->get_expiry_year(),
                    'checked' => '',
                );
                if ($token->is_default()) {
                    $prefillData[$k]['checked'] = 'checked';
                }
            }
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->prefillData: ' . print_r($prefillData, 1));
        }

        // Template Translations
        $translation = array(
            'Name on Card' => __('Card Holder', 'mipa'),
            'Credit Card Number' => __('Credit Card Number', 'mipa'),
            'Expiration Date' => __('Expiration Date', 'mipa'),
            'Card CVC' => __('CVC / CVV', 'mipa'),
            'Month' => __('Month', 'mipa'),
            'Year' => __('Year', 'mipa'),
            '#Jan#' => __('January', 'mipa'),
            '#Feb#' => __('February', 'mipa'),
            '#Mar#' => __('March', 'mipa'),
            '#Apr#' => __('April', 'mipa'),
            '#May#' => __('May', 'mipa'),
            '#Jun#' => __('Juni', 'mipa'),
            '#Jul#' => __('July', 'mipa'),
            '#Aug#' => __('August', 'mipa'),
            '#Sep#' => __('September', 'mipa'),
            '#Oct#' => __('October', 'mipa'),
            '#Nov#' => __('November', 'mipa'),
            '#Dec#' => __('December', 'mipa'),
            'Remember Data' => __('Remember card for future purchase', 'mipa'),
            'Use new Card' => __('Use new Card', 'mipa'),
            'Use old Card' => __('Use old Card', 'mipa'),
            'Holder' => __('Holder', 'mipa'),
            'CardNo' => __('CardNo', 'mipa'),
            'Expire' => __('Valid until', 'mipa'),
            'Remove' => __('Remove', 'mipa'),
            'Please check your inputs!' => __('Please check your inputs!', 'mipa'),
            'Vorname Nachname' => __('Firstname Lastname', 'mipa'),
            'NAME:' => __('Card Holder:', 'mipa'),
            'PAN:' => __('Card Number:', 'mipa'),
            'EXPIRE:' => __('Expire Date:', 'mipa'),
        );

        if ($addPaymethodPage) {
            $showRemember = FALSE;
            $prefillData = array();
        }
        $showRemove = FALSE;
        $options = array(
            'futureYears' => $this->futureYears,
            'preSeletActDate' => FALSE,
            'showMonthNames' => FALSE,
            'showRemember' => $showRemember,
            'showRemove' => $showRemove,
            'cardStyle' => TRUE,
            'cardSelect' => FALSE,
            //'imgPath'           => $this->plugin_url().'images/'
            'specialBrand' => TRUE,
            'specialHolder' => TRUE,
            //'preselectNewCard' => TRUE, // activate if you want new form as default
        );

        $formTPL = $this->payMiPa->getCCardTPL($filename, $options, $prefillData, $translation);
        //if ($this->debug) $this->log->debug(__CLASS__.'->'.__FUNCTION__.'->formTPL: '.print_r($formTPL,1));

        if ($this->description && !$addPaymethodPage) {
            echo apply_filters('wc_' . $this->id . '_description', $this->description);
        }

        // output form template
        if ($addPaymethodPage) {
            $output = __('Adding new card is only available in checkout.', 'mipa') . '<br>';
            $output .= __('Please save your card on next purchase.', 'mipa') . '<br><br>';
        } else {
            $output = $formTPL;
        }
        // start script
        $output .= '<script type="text/javascript">';
        // change form tag to have ankerpoint for script
        if ($onCheckoutPage) {
            $output .= 'document.getElementsByName("checkout")[0].setAttribute("id", "' . $this->payMiPa->prefixCCard . 'token-form");';
        } else {
            // dont change form, because checkout wont work anymore
            //$output.= 'document.getElementById("order_review").setAttribute("id", "' . $this->payMiPa->prefixCCard . 'token-form");';
        }
        // inline script
        $output .= $mipaJSInline;
        // end script
        $output .= '</script>';
        // load default style for fake fields
//        $output .= '<script src="' . $this->plugin_url() . 'mcpay/js/mcpay_formstyle_ccard_check.js"></script>';
//        $output .= '<link href="' . $this->plugin_url() . 'mcpay/css/mcpay_formstyle_ccard.css" rel="stylesheet" />';
        $output .= '<script src="' . $config['plugin_urljs'] . 'mcpay_formstyle_ccard_check"></script>';
        $output .= '<link href="' . $config['plugin_urlcss'] . 'mcpay_formstyle_ccard" rel="stylesheet" />';

        echo $output;
        //if ($this->debug) $this->log->debug(__CLASS__.'->'.__FUNCTION__.'->output: '.print_r($output,1));

        return TRUE;
    }

    /**
     * validate_fields
     * @return bool
     */
    function validate_fields()
    {
        global $woocommerce;
        //mail('test@test.test', __CLASS__.'->'.__FUNCTION__.' WC DEBUG', print_r($woocommerce,1));
        //mail('test@test.test', __CLASS__.'->'.__FUNCTION__.' GET DEBUG', print_r($_GET, 1));
        //mail('test@test.test', __CLASS__.'->'.__FUNCTION__.' POST DEBUG', print_r($_POST, 1));
        $error_msg = '';
        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->POST: ' . print_r($_POST, 1));

        if (!empty($_POST[$this->payMiPa->prefixCCard . 'reuse']) && $_POST[$this->payMiPa->prefixCCard . 'reuse'] != 'new') {
            // reuse old card
            $woocommerce->session->mipa_card_token = 'reuse';

        } else {
            // new card
            if (empty($_POST[$this->payMiPa->prefixCCard . 'token'])) {
                $error_msg = __('Please check your inputs!', 'mipa');

                // add payment method page
            } else if (!empty($_POST['woocommerce_add_payment_method'])) {
                return TRUE;

                // normal handling
            } else {
                $billingCountry = $woocommerce->customer->get_billing_country();
                if (!empty($_POST['billing_country'])) {
                    $billingCountry = $_POST['billing_country'];
                }

                $selectedPM = $_POST['payment_method'];
                $cardToken = $_POST[$this->payMiPa->prefixCCard . 'token'];

                if (!$this->payMiPa->isAllowedCountry($billingCountry, $selectedPM)) {
                    $error_msg = __('Payment not allowed in that country.', 'mipa');
                }

                $woocommerce->session->mipa_selected_pm = $selectedPM;
                $woocommerce->session->mipa_card_token = $cardToken;
                if (empty($woocommerce->session->mipa_card_token)) {
                    $error_msg = __('Payment Token cant be empty.', 'mipa');
                }

                $woocommerce->session->mipa_card_remember_card = FALSE;
                if (!empty($_POST[$this->payMiPa->prefixCCard . 'remember'])) {
                    $woocommerce->session->mipa_card_remember_card = TRUE;
                }
                $customerID = $woocommerce->customer->get_id();
                $payData = array(
                    'token' => $_POST[$this->payMiPa->prefixCCard . 'token'],
                    'holder' => $_POST[$this->payMiPa->prefixCCard . 'holder'],
                    'pan' => $_POST[$this->payMiPa->prefixCCard . 'pan'],
                    'month' => $_POST[$this->payMiPa->prefixCCard . 'month'],
                    'year' => $_POST[$this->payMiPa->prefixCCard . 'year'],
                    'cvc' => $_POST[$this->payMiPa->prefixCCard . 'cvc'],
                );
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->remember payData: ' . print_r($payData, 1) . ' Customer: ' . $customerID);
                $woocommerce->session->mipa_card_save_new_card = serialize($payData);

            }
        }

        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$woocommerce->session', print_r($woocommerce->session,1));
        
        if (!empty($error_msg)) {
            //$woocommerce->add_error(__('Error:', 'mipa').' '.$error_msg);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->error_msg: ' . $error_msg);
            wc_add_notice($error_msg, 'error');
            return FALSE;
        }
        return TRUE;
    }

    /**
     * add_payment_method
     * @return array
     */
    public function add_payment_method()
    {
        //mail('test@test.test', __CLASS__.'->'.__FUNCTION__.'DEBUG', print_r($_POST, 1));
        global $woocommerce;
        $customerID = $woocommerce->customer->get_id();
        $payData = array(
            'token' => $_POST[$this->payMiPa->prefixCCard . 'token'],
            'holder' => $_POST[$this->payMiPa->prefixCCard . 'holder'],
            'pan' => $_POST[$this->payMiPa->prefixCCard . 'pan'],
            'month' => $_POST[$this->payMiPa->prefixCCard . 'month'],
            'year' => $_POST[$this->payMiPa->prefixCCard . 'year'],
            'cvc' => $_POST[$this->payMiPa->prefixCCard . 'cvc'],
        );
        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->savePaymentToken payData: ' . print_r($payData, 1) . ' Customer: ' . $customerID);
        $this->savePaymentToken($payData, $customerID);

        return array(
            'result' => 'success',
            'redirect' => wc_get_endpoint_url('payment-methods'),
        );
    }

    /**
     * scheduled_subscription_payment
     *
     * @param $amount
     * @param $order
     * @param $product_id
     * @return bool|WP_Error
     */
    function scheduled_subscription_payment($amount, $order, $product_id = '')
    {
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__, print_r(func_get_args(),1).print_r($_POST,1).print_r($_GET,1));
        if (!$order instanceof WC_Order) {
            return new WP_Error($this->id . '_error', __('order not correct object.', 'mipa') . print_r($order, 1));
        }

        // check if saved card is still available
        $nextUserId = $order->get_user_id();
//        $tokens = $this->get_tokens($nextUserId);
//        if (empty($tokens)) {
//            $errorMsg = __('No saved card available.', 'mipa');
//            $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
//            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->get_tokens->error: ' . $errorMsg);
//            return new WP_Error(time(), $errorMsg);
//        }

        $config = $this->settings;
        $config['currency'] = get_woocommerce_currency();
        $config['card_token'] = 'reuse';

        // reuse a saved card
        try {
            $userData = $this->getUserDataFromOrder($order, $config);
            $userData['ip'] = '127.0.0.1'; // on renewal there is no customer ip
            $payData = array(
                'token' => 1,
                'reuse' => true,
            );
            $this->payMiPa->customPrefix = $config['project']; // set prefix to check older customer
            $res = $this->payMiPa->bookCCard($userData, $config, $payData);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->reuseCard->res: ' . print_r($res, 1));

            $trxID = $res->trxInfo->transactionId;
            $order->add_order_note(__('SCHEDULED TRX', 'mipa'));
            $order->set_transaction_id($trxID);
            $order->add_order_note(__('SESSIONID:', 'mipa') . $res->trxInfo->sessionId);
            $order->add_order_note(__('TRXID:', 'mipa') . $trxID);

            if (!empty($config['testMode'])) $order->add_order_note(__('!! TEST MODE !!', 'mipa'));
            $order_id = $order->get_id();

            // save important parameters
            $params = new stdClass();
            $params->sessionId = $res->trxInfo->sessionId;
            $params->transactionId = $res->trxInfo->transactionId;
            $this->saveOrderParams($order_id, $params); // save info to order

            if ($res->trxInfo->sessionStatus != 'SUCCESS') {
                $errorMsg = $res->trxInfo->transactionResultMessage;
                $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
                return new WP_Error($this->id . '_error', $errorMsg);
            }

            if (!empty($order)) {
                // mark order for processing
                $order->update_status('processing', __('Payment processed.', 'mipa'));

                WC_Subscriptions_Manager::process_subscription_payments_on_order( $order );

                return TRUE;
            }

        } catch (Exception $e) {
            $errorMsg = $e->getMessage();
            $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->reuseCard->error: ' . $errorMsg);

            WC_Subscriptions_Manager::process_subscription_payment_failure_on_order( $order, $product_id );

            return new WP_Error($this->id . '_error', $errorMsg);
        }

    }

    /**
     * process_payment
     * Process the payment and return the result
     * @access public
     *
     * @param int $order_id
     *
     * @return array
     */
    function process_payment($order_id)
    {
        global $woocommerce;
        $order = new WC_Order($order_id);
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$order', print_r($order,1));

        if (class_exists('WC_Subscriptions_Order')) {
            if (WC_Subscriptions_Order::order_contains_subscription($order_id)) {
                // always save card for reuse, otherwise subscription wont work
                $payData = $this->saveCardForReuse($order, true);
            }
        }

        $error_msg = '';
        if (empty($woocommerce->session->mipa_card_token)) {
            if (!empty($_POST[$this->payMiPa->prefixCCard . 'token'])) {
                $woocommerce->session->mipa_card_token = $_POST[$this->payMiPa->prefixCCard . 'token'];
                // remember for first booking
                $payData = array(
                    'token' => $_POST[$this->payMiPa->prefixCCard . 'token'],
                    'holder' => $_POST[$this->payMiPa->prefixCCard . 'holder'],
                    'pan' => $_POST[$this->payMiPa->prefixCCard . 'pan'],
                    'month' => $_POST[$this->payMiPa->prefixCCard . 'month'],
                    'year' => $_POST[$this->payMiPa->prefixCCard . 'year'],
                    'cvc' => $_POST[$this->payMiPa->prefixCCard . 'cvc'],
                );
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->remember payData: ' . print_r($payData, 1));
                $woocommerce->session->mipa_card_save_new_card = serialize($payData);
            } else {
                $error_msg = __('Payment Token cant be empty.', 'mipa');
            }
        }

        if (!empty($error_msg)) {
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->error_msg: ' . $error_msg);
            wc_add_notice($error_msg, 'error');
            return FALSE;
        }

        $config = $this->settings;
        $config['returnURL'] = add_query_arg('wc-api', 'wc_threedsecure_' . $this->id, home_url('/')) . '&orderID=' . $order_id;
        $config['returnURLNO3D'] = add_query_arg('wc-api', 'wc_nothreedsecure_' . $this->id, home_url('/')) . '&orderID=' . $order_id . '&sessID=';
//        $config['start3D'] = $this->start3D;
//        $config['start3Dreuse'] = $this->start3Dreuse;
        $config['card_token'] = $woocommerce->session->mipa_card_token;
        $config['plugin_path'] = $this->plugin_path();
        $config['plugin_url'] = $this->plugin_url();
//        $config['plugin_urljs'] = $this->plugin_url() . 'js/';
//        $config['plugin_urlcss'] = $this->plugin_url() . 'css/';
        $config['plugin_urljs'] = '?wc-api=wc_getjs_mipa_ccard&sn=';
        $config['plugin_urlcss'] = '?wc-api=wc_getcss_mipa_ccard&sn=';
        //$config['create_new_session'] = true;

        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->config: ' . print_r($config, 1));


        // first use of a card
        try {

            if (!empty($woocommerce->session->mipa_card_save_new_card)) {
                $payData = $woocommerce->session->mipa_card_save_new_card;
                $payData = unserialize($payData);
            }
//            if (empty($payData['token']) && !empty($woocommerce->session->mipa_thereWasAnErrorBefore)){
//                $customerID = $order->get_customer_id();
//                $this->deletePaymentToken($customerID);
//                wc_add_notice('Invalid Paymethod Data. Card removed!', 'error');
//                $woocommerce->session->mipa_thereWasAnErrorBefore = false;
//                return array(
//                    'result' => 'success',
//                    'redirect' => wc_get_checkout_url()
//                );
//            }

            $userData = $this->getUserDataFromOrder($order, $config);
            if ($woocommerce->session->mipa_card_token == 'reuse') {
                $payData['reuse'] = TRUE;
                $payData['token'] = TRUE;
//                $res = $this->payMiPa->reuseCard($userData, $config);
//                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->reuseCard->res: ' . print_r($res, 1));
            } else {
                $payData['reuse'] = FALSE;
//                $res = $this->payMiPa->bookCCard($userData, $config, $payData);
//                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->bookCCard->res: ' . print_r($res, 1));
            }
            $res = $this->payMiPa->bookCCard($userData, $config, $payData);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->bookCCard->res: ' . print_r($res, 1));

            if ($res->trxInfo->secureStatus != 'START3D') {
                // save important parameters
                $params = new stdClass();
                $params->sessionId = $res->trxInfo->sessionId;
                $params->transactionId = $res->trxInfo->transactionId;
                $this->saveOrderParams($order_id, $params); // save info to order
            }

            if (!empty($res->forward)) {
                return array(
                    'result' => 'success',
                    'redirect' => $res->forwardURL
                );
            }

        } catch (Exception $e) {
            $errorMsg = __($e->getMessage(), 'mipa'); // try to translate error
            $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
            wc_add_notice($errorMsg, 'error');
            // delete card in case of error
            $customerID = $order->get_customer_id();
            $this->deletePaymentToken($customerID);

            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->bookCCard->error: ' . $errorMsg);
            $woocommerce->session->mipa_thereWasAnErrorBefore = true;
            return FALSE;
        }

        return array(
            'result' => 'success',
            'redirect' => $order->get_checkout_payment_url(TRUE)
        );


    }

    /**
     * responseAction
     * handle response
     * @throws Exception
     */
    function responseAction()
    {
        global $woocommerce;
        $crlf = "\n";
        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->GET: ' . print_r($_GET, 1) . '->POST: ' . print_r($_POST, 1));
        $return = 'status=SUCCESS' . $crlf;

        $sessID = $_REQUEST['sessionId'];
        $p = explode('_', $sessID);
        $orderId = $p[1];
        $custID = $_REQUEST['customerId'];
        $p = explode('_', $custID);
        $customerId = $p[1];
        $order_id = $orderId;

        if (empty($orderId) && !empty($_GET['orderid'])) {
            $orderId = $_GET['orderid'];
            $p = explode('_', $orderId);
            $orderId = $p[1];
            $order = new WC_Order($orderId);
            $customerId = $order->get_customer_id();
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->orderA: ' . $orderId . print_r($order, 1));
        } else if (empty($orderId) && !empty($_REQUEST['freeParam']['orderid'])) {
            $orderId = $_REQUEST['freeParam']['orderid'];
            if (strpos($orderId, '_') !== false) {
                $p = explode('_', $orderId);
                $orderId = $p[1];
            }
            $order = new WC_Order($orderId);
            $customerId = $order->get_customer_id();
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->orderB: ' . $orderId . print_r($order, 1));
        } else if (!empty($orderId)) {
            $order = new WC_Order($orderId);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->orderC: ' . $orderId . print_r($order, 1));
        } else {
            $return = 'status=ERROR' . $crlf . 'message=OrderID missing.' . $crlf;
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->return: ' . print_r($return, 1));
            echo $return;
            exit();
        }

        $func = '';
        if (!empty($_REQUEST['function'])) {
            $func = strtolower($_REQUEST['function']);
        } else if (!empty($_REQUEST['action'])) {
            $func = $_REQUEST['action'];
        } else {
            $return = 'status=ERROR' . $crlf . 'message=Function missing.' . $crlf;
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->return: ' . print_r($return, 1));
            echo $return;
            exit();
        }

        switch ($func) {
            // debug info
            case 'custinfo':
                $config = $this->settings;
                if ($_GET['ak'] == $config['accessKey']) {
                    echo '<pre>' . print_r($config, 1) . '</pre>';
                    $order = new WC_Order($order_id);
                    echo '<pre>' . print_r($order, 1) . '</pre>';
                    $userId = $order->get_user_id();
                    $tokens = $this->get_tokens($userId);
                    echo '<pre>' . print_r($tokens, 1) . '</pre>';
                    $info = $this->loadOrderParams($order_id);
                    echo '<pre>' . print_r($info, 1) . '</pre>';
                } else {
                    echo 'Not found!';
                }
                exit();
                break;

            case 'getToken':
                $tokens = WC_Payment_Tokens::get_tokens(array('gateway_id' => $this->id));
                echo '<pre>'.print_r($tokens,true).'</pre>';
                exit();
                break;

            case 'rmToken':
                $tokens = WC_Payment_Tokens::get_tokens(array('gateway_id' => $this->id));
                echo '<pre>'.print_r($tokens,true).'</pre>';
                foreach ($tokens AS $id => $token) {
                    WC_Payment_Tokens::delete($id);
                }
                exit();
                break;

            case 'error':
                if ($this->error_page == 'CART') {
                    $url = $order->get_cancel_order_url(); // cancel order url
                } else {
                    $url = wc_get_checkout_url(); // checkout url
                }

                $return = 'status=SUCCESS' . $crlf // ok  | error
                    . 'url=' . $url . $crlf
                    . 'target=_top' . $crlf
                    . 'forward=1' . $crlf
                    . 'message=OK' . $crlf;
                break;
            case 'billing':
                if ($order->get_status() !== 'completed') {
                    if ($order->get_total() != ($_GET['amount'] / 100)) {
                        $return = 'status=ERROR' . $crlf . 'message=Amount mismatch.' . $crlf;
                        if ($this->debug) $this->log->error(__CLASS__ . '->' . __FUNCTION__ . '->error: Order(' . $orderId . ') Amount mismatch ' . $order->get_total() . ' != ' . ($_GET['amount'] / 100));
                    } else if ($order->get_currency() != $_GET['currency']) {
                        $return = 'status=ERROR' . $crlf . 'message=Currency mismatch.' . $crlf;
                        if ($this->debug) $this->log->error(__CLASS__ . '->' . __FUNCTION__ . '->error: Order(' . $orderId . ') Currency mismatch ' . $order->get_currency() . ' != ' . $_GET['currency']);
                    } else if ($order->get_customer_id() != $customerId) {
                        $return = 'status=ERROR' . $crlf . 'message=Customer mismatch.' . $crlf;
                        if ($this->debug) $this->log->error(__CLASS__ . '->' . __FUNCTION__ . '->error: Order(' . $orderId . ') Customer mismatch ' . $order->get_customer_id() . ' != ' . $customerId);
                    } else {
                        if (!empty($_GET['testmode'])) {
                            $order->add_order_note(__('!! TEST MODE !!', 'mipa'));
                        }

                        // redo in case of thanksAction didnt fire!
                        $trxID = $_GET['transactionId'];
                        $order->add_order_note(__('TRXID:', 'mipa') . $trxID);
                        $order->add_order_note(__('SESSIONID:', 'mipa') . $_GET['sessionId']);
                        $order->set_transaction_id($_GET['transactionId']);
                        $order->save();

                        $order->payment_complete();
                        $config = $this->settings;
                        if (!$this->isOrderStatus($order->get_status(), $config['status_paid'])) {
                            $order->set_status($config['status_paid']);
                            $order->save();
                        }

                        $return = 'status=SUCCESS' . $crlf // ok  | error
                            . 'url=' . $this->get_return_url($order) . $crlf
                            . 'target=_top' . $crlf
                            . 'forward=1' . $crlf
                            . 'message=OK' . $crlf;
                    }
                }
                break;
            case 'storno': // chargeback
            case 'transactionChargeback':
                $order->update_status('cancelled', __('Payment Cancelled. (STORNO)', 'mipa'));
                break;
            case 'backpay': // chargeback reversal
            case 'transactionBackpay':
                $order->update_status('refunded', __('Payment Cancellation refunded (BACKPAY)', 'mipa'));
                break;
            case 'refund': // refund
            case 'transactionRefund':
                // partial refund
                if (($order->get_total() * 100) != $_REQUEST['amount']) {
                    $amount = ($_REQUEST['amount'] / 100);
                    $refund = wc_create_refund(
                        array(
                            'amount' => $amount,
                            'reason' => __('Payment Refund', 'mipa'),
                            'order_id' => $orderId
                        )
                    );
                    if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->refund: ' . print_r($refund, 1));
                    if (is_a($refund, 'WP_Error')) {
                        $order->add_order_note(__('Refund Error ', 'mipa') . $amount . ' ' . $_REQUEST['currency'] . ' ' . $refund->get_error_message());
                        $return = 'status=ERROR' . $crlf . 'message=' . $refund->get_error_message() . $crlf;
                    } else {
                        $order->add_order_note(__('Partial Refund ', 'mipa') . $amount . ' ' . $_REQUEST['currency']);
                    }

                } else {
                    // full refund
                    $order->update_status('refunded', __('Payment Refund (REFUND)', 'mipa'));
                }
                break;
            case 'quit': // reversal SEPA only
                $order->update_status('cancelled', __('Payment Cancelled. (QUIT)', 'mipa'));
                break;
        }
        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->return: ' . print_r($return, 1));
        echo $return;
        exit();
    }

    /**
     * threedsecureAction
     *
     */
    function threedsecureAction()
    {
        $crlf = "\r\n";
        global $woocommerce;
        if (!empty($_GET['orderID'])) {
            $order_id = (int)$_GET['orderID'];
            $order = new WC_Order($order_id);
        } else {
            echo 'OrderID missing!';
        }

        if (!empty($_REQUEST['check3dId'])) {
            $check3dId = $_REQUEST['check3dId'];
        } else {
            echo '3D Secure failed!';
            exit();
        }

        try {
            $config = $this->settings;
            $this->payMiPa->setConfig($config);

            $res = $this->payMiPa->verify3D($check3dId, $config);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->verify3D: ' . $check3dId . print_r($res, 1));

            if (isset($res->error) && $res->error == 0) {
                // save important parameters
                $params = new stdClass();
                $params->sessionId = $res->sessionId;
                $params->transactionId = $res->transactionId;
                $this->saveOrderParams($order_id, $params); // save info to order

                $trxID = $res->transactionId;
                $sessID = $res->sessionId;
                $order->add_order_note(__('TRXID:', 'mipa') . $trxID);
                $order->add_order_note(__('SESSIONID:', 'mipa') . $sessID);
                $order->set_transaction_id($trxID);
                $order->save();

                $this->saveCardForReuse($order, false); // save for reuse if checked

                $order->payment_complete();
                $config = $this->settings;
                if (!$this->isOrderStatus($order->get_status(), $config['status_paid'])) {
                    $order->set_status($config['status_paid']);
                    $order->save();
                }
                $url = $this->get_return_url($order);
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->verify3D->redirect: ' . $url);
                wp_redirect($url);
                return;
            }
        } catch (Exception $e) {
            $errorMsg = __($e->getMessage(), 'mipa'); // try to translate error
            $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
            wc_add_notice($errorMsg, 'error');
            // delete card in case of error
            $customerID = $order->get_customer_id();
            $this->deletePaymentToken($customerID);

            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->verify3D->error: ' . $errorMsg);
            $order->save();
        }

        if ($this->error_page == 'CART') {
            $url = wc_get_cart_url(); // cart url
            $url = $order->get_cancel_order_url(); // cancel order url
        } else {
            $url = wc_get_checkout_url(); // checkout url
        }
        wp_redirect($url);
    }

    function nothreedsecureAction()
    {
        global $woocommerce;

        if (!empty($_GET['orderID'])) {
            $order_id = (int)$_GET['orderID'];
            $order = new WC_Order($order_id);
        } else {
            echo 'OrderID missing!';
            exit();
        }

        try {
            $config = $this->settings;
            $this->payMiPa->setConfig($config);

            if (!empty($_GET['sessID'])) {
                $sessID = $_GET['sessID'];
            } else {
                echo 'SessionID missing!';
                exit();
            }

            $res = $this->payMiPa->sessionGet($sessID);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->sessionGet: ' . $sessID . print_r($res, 1));

            if (isset($res->error) && $res->error == 0) {
                $trxID = $res->transactionIds[0];
                $order->add_order_note(__('TRXID:', 'mipa') . $trxID);
                $order->add_order_note(__('SESSIONID:', 'mipa') . $sessID);
                $order->set_transaction_id($trxID);
                $order->save();

                $this->saveCardForReuse($order, false, $res->customerId); // save for reuse if checked

                $order->payment_complete();
                $config = $this->settings;
                if (!$this->isOrderStatus($order->get_status(), $config['status_paid'])) {
                    $order->set_status($config['status_paid']);
                    $order->save();
                }
                $url = $this->get_return_url($order);
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->get_return_url->wp_redirect: ' . $url);
                wp_redirect($url);
                return;
            }
        } catch (Exception $e) {
            $errorMsg = __($e->getMessage(), 'mipa'); // try to translate error
            $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
            wc_add_notice($errorMsg, 'error');
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->sessionGet->error: ' . $errorMsg);
            $order->save();
        }

        if ($this->error_page == 'CART') {
            $url = wc_get_cart_url(); // cart url
            $url = $order->get_cancel_order_url(); // cancel order url
        } else {
            $url = wc_get_checkout_url(); // checkout url
        }
        wp_redirect($url);
    }

    /**
     * process_refund
     *
     * @param int $order_id
     * @param null $amount
     * @param string $reason
     *
     * @return bool|\WP_Error
     */
    public function process_refund($order_id, $amount = null, $reason = '')
    {
        $order = new WC_Order($order_id);

        $config = $this->settings;
        $config['amount'] = $amount;
        $config['currency'] = get_woocommerce_currency();
        $config['trxid'] = $this->getTrxID($order_id);
        $config['sessid'] = $this->getSessID($order_id);

        if (empty($config['amount']) || $config['amount'] <= 0) {
            return new WP_Error(time(), __('Amount cant be empty!', 'mipa'));
        }
        if (empty($config['currency'])) {
            return new WP_Error(time(), __('Currency cant be empty!', 'mipa'));
        }
        if (empty($config['trxid'])) {
            return new WP_Error(time(), __('TRXID cant be empty!', 'mipa'));
        }
        if (empty($config['sessid'])) {
            return new WP_Error(time(), __('SessionID cant be empty!', 'mipa'));
        }

        try {
            $refundInfo = $this->payMiPa->refundTRX($order, $config);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->refundTRX: ' . print_r($refundInfo, 1));
            $this->saveOrderParams($order_id, $refundInfo, '_mipa_refund_params'); // save refund info to order
            if ($refundInfo->transactionStatus == 'SUCCESS') {
                return TRUE;
            }
        } catch (Exception $e) {
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->refundTRX->error: ' . $e->getMessage());
            return new WP_Error(time(), $e->getMessage());
        }

        return new WP_Error(time(), __('Unknown Refund Error.', 'mipa'));
    }

    /**
     * saveCardForReuse
     *
     * @param $order
     * @param bool $forceSave
     * @param null $custID
     * @return bool|mixed
     */
    function saveCardForReuse($order, $forceSave = false, $custID = NULL)
    {
        global $woocommerce;
        // Save Card for reuse
        if (!empty($woocommerce->session->mipa_card_save_new_card)) {
            $customerID = $order->get_customer_id();
            if (empty($customerID) && !empty($custID)){
                $customerID = $custID;
            }
            $payData = $woocommerce->session->mipa_card_save_new_card;
            $woocommerce->session->mipa_card_save_new_card = ''; // remove from session
            $woocommerce->session->mipa_card_remember_card = FALSE;
            $payData = unserialize($payData);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->saved unserialized payData: ' . print_r($payData, 1) . ' Customer: ' . $customerID);

            $this->savePaymentToken($payData, $customerID);
            return $payData;
        }

        return array();
    }

    /**
     * savePaymentToken
     * saves the payment token to wc payment tokens
     * @param $payData
     * @param $customerID
     */
    function savePaymentToken($payData, $customerID)
    {
        // remove old token
        $tokens = $this->get_tokens($customerID);
        foreach ($tokens AS $k => $token) {
            $payToken = new WC_Payment_Token_CC($token->get_id());
            $payToken->delete();
        }
        // save new token
        $payToken = new WC_Payment_Token_CC();
        $payToken->set_token($payData['token']);
        $payToken->set_gateway_id($this->id);
        $payToken->set_user_id($customerID);
        $payToken->set_expiry_month(sprintf('%02d', $payData['month']));
        $payToken->set_expiry_year(($payData['year'] + 2000));
        $payToken->set_last4($payData['pan']);
        $cardType = 'visa';
        if (substr($payData['pan'], 0, 1) == '5') $cardType = 'mastercard';
        $payToken->set_card_type($cardType . ' (' . $payData['holder'] . ')');
        //$payToken->set_default();
        $payToken->save();
        //$payToken->save_meta_data();
        WC_Payment_Tokens::set_users_default($customerID, $payToken->get_id());
    }

    /**
     * deletePaymentToken
     *
     * @param $customerID
     */
    function deletePaymentToken($customerID)
    {
        // remove old token
        $tokens = $this->get_tokens($customerID);
        foreach ($tokens AS $k => $token) {
            $payToken = new WC_Payment_Token_CC($token->get_id());
            $payToken->delete();
        }
    }

}

?>