<?php

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class WC_Gateway_Mipa_DD extends WC_Gateway_Mipa_Base
{

    const id = 'mipa_sepa';

    /**
     * Constructor for the gateway.
     * @access public
     * @return void
     */
    public function __construct()
    {
        $this->id = self::id;
        $this->method_title = __('Micropayment SEPA', 'mipa');
        $this->has_fields = TRUE;

        // Load the settings
        $this->init_form_fields();
        $this->init_settings();

        // Actions
        add_action('woocommerce_api_wc_gateway_' . $this->id, array($this, 'responseAction'));
        add_action('woocommerce_email_after_order_table', array($this, 'addContentToEMail'), 20, 4);
        add_action('woocommerce_thankyou_' . $this->id, array($this, 'addContentThankyou'));

        parent::__construct();
    }

    /**
     * is_valid_for_use
     *
     * @return bool
     * @throws \Exception
     */
    public function is_valid_for_use()
    {
        global $woocommerce;

        $onCheckoutPage = TRUE;
        if (!is_checkout()) $onCheckoutPage = FALSE;
        if (is_checkout_pay_page()) $onCheckoutPage = FALSE;

        if (!empty($_GET['order-pay'])) {
            $orderId = (int)$_GET['order-pay'];
            $order = new WC_Order($orderId);

            // check if paymethod is mipa_sepa and not in checkout
            if ($order->payment_method == 'mipa_sepa' && !$onCheckoutPage) {
                $config = $this->settings;
                $this->payMiPa->setConfig($config);

                $opObj = $this->loadOrderParams($orderId);
                $sessionId = $opObj->bookObj->orderID;
                if (!empty($sessionId)) {
                    $this->payMiPa->switchPaymethod('sepa');
                    try {
                        $sessionInfo = $this->payMiPa->sessionGet($sessionId);
                        // if chargeback do not allow to reuse sepa
                        if ($sessionInfo->status == 'REVERSED') {
                            return FALSE;
                        }
                    } catch (Exception $e) {
                        // if no session outside checkout disallow sepa
                        return FALSE;
                    }
                }
            }
        }

        return parent::is_valid_for_use();
    }

    /**
     * Initialise Gateway Settings Form Fields
     * @access public
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable/Disable', 'mipa'),
                'type' => 'checkbox',
                'label' => __('Enable Plugin', 'mipa'),
                'default' => 'yes'
            ),
            'title_de' => array(
                'title' => __('Title', 'mipa').' DE',
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'mipa'),
                'default' => 'Micropayment SEPA Lastschrift',
                'desc_tip' => TRUE,
            ),
            'title_en' => array(
                'title' => __('Title', 'mipa').' EN',
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'mipa'),
                'default' => 'Micropayment SEPA Direct Debit',
                'desc_tip' => TRUE,
            ),
            'description_de' => array(
                'title' => __('Description', 'mipa').' DE',
                'type' => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'mipa'),
                'default' => $this->get_poweredby() . '<b>Bezahlung per SEPA Lastschrift</b><br>Mit diesem einfachen und sicheren Bezahlsystem können Sie direkt mit Ihrer IBAN bezahlen.<br><br>'
            ),
            'description_en' => array(
                'title' => __('Description', 'mipa').' EN',
                'type' => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'mipa'),
                'default' => $this->get_poweredby() . '<b>Payment with SEPA direct debit</b><br>With this simple and secure payment system, you can pay directly with your IBAN.<br><br>'
            ),

            // BIC: INGDDEFFXXX IBAN: DE12500105170648489890
            'mini_amount' => array(
                'title' => __('Min Amount', 'mipa'),
                'description' => __('Setup the minimum amount for this paymethod. In Euro CENT', 'mipa'),
                'type' => 'text',
                'default' => '99',
                'desc_tip' => TRUE,
            ),
            'maxi_amount' => array(
                'title' => __('Max Amount', 'mipa'),
                'description' => __('Setup the maximum amount for this paymethod. 0 means no Limit! In Euro CENT. In case you need a higher limit, please contact us.', 'mipa'),
                'type' => 'text',
                'default' => '50000',
                'desc_tip' => TRUE,
            ),
            'freetext' => array(
                'title' => __('Freetext on settlement', 'mipa'),
                'description' => __('Here you can define a freetext for the customer settlement.', 'mipa').' '.__('Placeholder {$orderid} = OrderID', 'mipa'),
                'type' => 'text',
                'placeholder' => __('Order {$orderid} Your Freetext!!', 'mipa'),
                'default' => '',
                'desc_tip' => __('If empty only order id will be used.', 'mipa'),
            ),
            'paydate' => array(
                'title' => __('Pay in days', 'mipa'),
                'description' => __('In how many days the payment should be executed.', 'mipa'),
                'type' => 'text',
                'default' => '7',
                'desc_tip' => TRUE,
            ),
            'mandat_text_de' => array(
                'title' => __('Mandate Text German', 'mipa'),
                'desc_tip' => __('Setup the mandate text in german', 'mipa'),
                'type' => 'textarea',
                'default' => __('Unsere Forderung in Höhe von {$amount} {$currency} ziehen wir mit der SEPA-Lastschrift zum Mandat Nr. {$mandateRef} '
                    . 'zu der Gläubiger-Identifikationsnummer {$creditorId} von Ihrem Konto IBAN {$iban} bei {$bankname} BIC {$bic} '
                    . 'zum Fälligkeitstag {$payDateDE}{$payRecurSt} ein. Wir bitten Sie für Kontodeckung zu sorgen.', 'mipa'),
                'description' => 'Die folgenden Platzhalter sind verfügbar:<br>'
                    . '{$amount} - Buchungsbetrag<br>'
                    . '{$currency} - Buchungswährung<br>'
                    . '{$mandateRef} - Mandats Referenz<br>'
                    . '{$creditorId} - Creditor ID<br>'
                    . '{$iban} - IBAN<br>'
                    . '{$bankname} - Bank Name<br>'
                    . '{$bic} - BIC<br>'
                    . '{$payDateDE} - Datum<br>'
                    . '{$payRecurSt} - Status Wiederholungsbuchung<br>',
            ),
            'mandat_text_en' => array(
                'title' => __('Mandate Text English', 'mipa'),
                'desc_tip' => __('Setup the mandate text in english', 'mipa'),
                'type' => 'textarea',
                'default' => __('Please confirm the direct debit authorization. '
                    . 'We will deduct the payment amounting to {$amount} {$currency} using SEPA direct debit on the day it is due {$payDateEN}. '
                    . 'Please ensure that your account has sufficient funds.', 'mipa'),
                'description' => 'The following placeholder are available:<br>'
                    . '{$amount} - Amount to book<br>'
                    . '{$currency} - Currency to book<br>'
                    . '{$mandateRef} - Mandate Ref<br>'
                    . '{$creditorId} - Creditor ID<br>'
                    . '{$iban} - IBAN<br>'
                    . '{$bankname} - Bank Name<br>'
                    . '{$bic} - BIC<br>'
                    . '{$payDateEN} - Date<br>'
                    . '{$payRecurSt} - Recurring State<br>',
            ),
        );
        $baseFields = $this->get_base_form_fields();
        unset($baseFields['status_pending']); // prepay only
        $this->form_fields = array_merge($baseFields, $this->form_fields);
    }

    /**
     * payment_fields
     * @return bool
     */
    function payment_fields()
    {
        global $woocommerce;
        $config = $this->settings;
        $config['plugin_path'] = $this->plugin_path();
        $config['plugin_url'] = $this->plugin_url();
        $config['plugin_urljs'] = '?wc-api=wc_getjs_mipa_ccard&sn=';
        $config['plugin_urlcss'] = '?wc-api=wc_getcss_mipa_ccard&sn=';

        $onCheckoutPage = TRUE;
        if (!is_checkout()) $onCheckoutPage = FALSE;
        if (is_checkout_pay_page()) $onCheckoutPage = FALSE;

        $addPaymethodPage = TRUE;
        if (!is_add_payment_method_page()) $addPaymethodPage = FALSE;

        $this->payMiPa->setConfig($config);

        $filename = $config['plugin_path'] . 'mcpay/view/mcpay_form_sepa.tpl';
        $showRemember = TRUE;
        $prefillData = array();
        $customerID = $woocommerce->customer->get_id();
        if (empty($customerID)) {
            // guest or new customer
            $showRemember = FALSE;
        } else {
            // load saved paydata old
            $tokens = $this->get_tokens($customerID);
            foreach ($tokens AS $k => $token) {
                $tmp = unserialize($token->get_token());
                $prefillData[$k] = array(
                    'filldesc' => $token->get_display_name(),
                    'holder' => $tmp['holder'],
                    'iban' => $tmp['iban'],
                    'bic' => $tmp['bic'],
                    'checked' => '',
                );
                if ($token->is_default()) {
                    $prefillData[$k]['checked'] = 'checked';
                }
            }
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->prefillData: ' . print_r($prefillData, 1));
        }

        // Template Translations
        $translation = array(
            'Name on Card' => __('Account Holder', 'mipa'),
            'Remember Data' => __('Remember Account for future purchase', 'mipa'),
            'Use new Card' => __('Use new Account', 'mipa'),
            'Use old Card' => __('Use old Account', 'mipa'),
            'Holder' => __('Holder', 'mipa'),
            'IBAN' => __('IBAN', 'mipa'),
            'BIC' => __('BIC', 'mipa'),
            'Remove' => __('Remove', 'mipa'),
            'Please check your inputs!' => __('Please check your inputs!', 'mipa'),
            'Vorname Nachname' => __('Firstname Lastname', 'mipa'),
        );

        if ($addPaymethodPage) {
            $showRemember = FALSE;
            $prefillData = array();
        }
        $showRemove = FALSE;
        $options = array(
            'showRemember' => $showRemember,
            'showRemove' => $showRemove,
            'cardStyle' => TRUE,
            //'imgPath' => $this->plugin_url() . 'images/'
        );

        $formTPL = $this->payMiPa->getSepaTPL($filename, $options, $prefillData, $translation);

        if ($this->description && !$addPaymethodPage) {
            echo apply_filters('wc_' . $this->id . '_description', $this->description);
        }

        // output form template
        $output = $formTPL;

        // load style and js
        $output .= '<script src="' . $config['plugin_urljs'] . 'mcpay_formstyle_sepa"></script>';
        $output .= '<link href="' . $config['plugin_urlcss'] . 'mcpay_formstyle_sepa" rel="stylesheet" />';

        echo $output;
        return TRUE;
    }

    /**
     * validate_fields
     * @return bool
     */
    function validate_fields()
    {
        global $woocommerce;
        $error_msg = '';
        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->POST: ' . print_r($_POST, 1));

        $woocommerce->session->mipa_sepa_token = 'new';
        if (!empty($_POST[$this->payMiPa->prefixSepa . 'reuse']) && $_POST[$this->payMiPa->prefixSepa . 'reuse'] != 'new') {
            // reuse old card
            $woocommerce->session->mipa_sepa_token = 'reuse';
            $woocommerce->session->mipa_sepa_tokenID = $_POST[$this->payMiPa->prefixSepa . 'reuse'];

        } else {
            // check mandatory fields
            $iban = $bic = '';
            if (!empty($_POST[$this->payMiPa->prefixSepa . 'iban']))
                $iban = strtoupper($_POST[$this->payMiPa->prefixSepa . 'iban']);
            if (!empty($_POST[$this->payMiPa->prefixSepa . 'bic']))
                $bic = strtoupper($_POST[$this->payMiPa->prefixSepa . 'bic']);

            if (empty($_POST[$this->payMiPa->prefixSepa . 'holder'])) {
                $error_msg = __('Holder cant be empty!', 'mipa');
            } else if (empty($iban)) {
                $error_msg = __('IBAN cant be empty!', 'mipa');
            } else if (substr($iban, 0, 2) != 'DE' && empty($bic)) { // bic empty and iban not DE
                $error_msg = __('BIC cant be empty!', 'mipa');

                // add payment method page
            } else if (!empty($_POST['woocommerce_add_payment_method'])) {
                return TRUE;

                // normal handling
            } else {
                $billingCountry = $woocommerce->customer->get_billing_country();
                if (!empty($_POST['billing_country'])) {
                    $billingCountry = $_POST['billing_country'];
                }

                $selectedPM = $_POST['payment_method'];
                if (!$this->payMiPa->isAllowedCountry($billingCountry, $selectedPM)) {
                    $error_msg = __('Payment not allowed in that country.', 'mipa');
                }

                $woocommerce->session->mipa_selected_pm = $selectedPM;

                $woocommerce->session->mipa_card_remember_card = FALSE;
                if (!empty($_POST[$this->payMiPa->prefixSepa . 'remember'])) {
                    $woocommerce->session->mipa_card_remember_card = TRUE;
                }
                $customerID = $woocommerce->customer->get_id();
                $payData = array(
                    'holder' => $_POST[$this->payMiPa->prefixSepa . 'holder'],
                    'iban' => $iban,
                    'bic' => $bic,
                );
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->remember payData: ' . print_r($payData, 1) . ' Customer: ' . $customerID);
                $woocommerce->session->mipa_card_save_new_card = serialize($payData);

            }
        }

        if (!empty($error_msg)) {
            //$woocommerce->add_error(__('Error:', 'mipa').' '.$error_msg);
            wc_add_notice($error_msg, 'error');
            return FALSE;
        }
        return TRUE;
    }

    /**
     * add_payment_method
     * @return array
     */
    public function add_payment_method()
    {
        global $woocommerce;
        $customerID = $woocommerce->customer->get_id();
        $payData = array(
            'holder' => $_POST[$this->payMiPa->prefixSepa . 'holder'],
            'iban' => $_POST[$this->payMiPa->prefixSepa . 'iban'],
            'bic' => $_POST[$this->payMiPa->prefixSepa . 'bic'],
        );
        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->savePaymentToken payData: ' . print_r($payData, 1) . ' Customer: ' . $customerID);
        try {
            $this->savePaymentToken($payData, $customerID);
        } catch (Exception $e) {
            $errorMsg = $e->getMessage();
            wc_add_notice($errorMsg, 'error');
            return array('result' => 'error');
        }

        return array(
            'result' => 'success',
            'redirect' => wc_get_endpoint_url('payment-methods'),
        );
    }

    /**
     * scheduled_subscription_payment
     *
     * @param $amount
     * @param $order
     * @param string $product_id
     * @return bool|WP_Error
     */
    function scheduled_subscription_payment($amount, $order, $product_id = '')
    {
        if (!$order instanceof WC_Order) {
            return new WP_Error($this->id . '_error', __('order not correct object.', 'mipa') . print_r($order, 1));
        }

        // check if saved card is still available
        $nextUserId = $order->get_user_id();
        $config = $this->settings;
        $config['currency'] = get_woocommerce_currency();
        $config['card_token'] = 'reuse';
        $config['isAbo'] = TRUE;

        // reuse a saved card
        try {
            // get paydata from MCP to use last used bank account
            $payData = array();
            $payData['reuse'] = true;
            $this->payMiPa->setConfig($config);
            $this->payMiPa->switchPaymethod('sepa');

            if (!empty($config['testMode'])) $order->add_order_note(__('!! TEST MODE !!', 'mipa'));
            $order_id = $order->get_id();

            $userData = $this->getUserDataFromOrder($order, $config);
            $userData['ip'] = '127.0.0.1'; // on renewal there is no customer ip
            $this->payMiPa->customPrefix = $config['project']; // set prefix to check older customer
            $res = $this->payMiPa->bookSepaShort($userData, $config, $payData);

            if ($res->resApprove->status == 'APPROVED') {
                $order->add_order_note(__('PAYTOKEN:', 'mipa') . $res->resCreate->payToken);
                $order->add_order_note(__('SESSIONID:', 'mipa') . $res->resCreate->sessionId);

                // save important parameters
                $params = new stdClass();
                $params->sessionId = $res->resCreate->sessionId;
                $params->transactionId = $res->resCreate->payToken;
                $this->saveOrderParams($order_id, $params); // save info to order

                $lang = strtolower($order->get_billing_country());
                $order->set_transaction_id($res->resCreate->payToken);
                $order->save();
            }

            if (!empty($order)) {
                // mark order for processing
                $order->update_status($config['status_paid'], __('Payment processed.', 'mipa'));

                WC_Subscriptions_Manager::process_subscription_payments_on_order( $order );
                return TRUE;
            }

        } catch (Exception $e) {
            $errorMsg = $e->getMessage();
            $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->reuseCard->error: ' . $errorMsg);

            WC_Subscriptions_Manager::process_subscription_payment_failure_on_order( $order, $product_id );
            return new WP_Error($this->id . '_error', $errorMsg);
        }

    }

    /**
     * process_payment
     * Process the payment and return the result
     * @access public
     *
     * @param int $order_id
     *
     * @return array
     */
    function process_payment($order_id)
    {
        global $woocommerce;
        $order = new WC_Order($order_id);

        $mipa_is_abo = FALSE;
        if (class_exists('WC_Subscriptions_Order')) {
            if (WC_Subscriptions_Order::order_contains_subscription($order_id)) {
                $mipa_is_abo = TRUE;
                // always save card for reuse, otherwise subscription wont work
                if (!empty($woocommerce->session->mipa_card_save_new_card)) {
                    $payData = $woocommerce->session->mipa_card_save_new_card;
                    $woocommerce->session->mipa_card_save_new_card = ''; // remove from session
                    $woocommerce->session->mipa_card_remember_card = TRUE; // force save after success trx for subscription
                    $payData = unserialize($payData);
                }
            }
        }

        // Save Card for reuse
        if ($woocommerce->session->mipa_card_remember_card && !empty($woocommerce->session->mipa_card_save_new_card)) {
            if (empty($payData)) {
                $payData = $woocommerce->session->mipa_card_save_new_card;
                $woocommerce->session->mipa_card_save_new_card = ''; // remove from session
                $payData = unserialize($payData);
            }
        }

        $config = $this->settings;
        $config['plugin_path'] = $this->plugin_path();
        $config['plugin_url'] = $this->plugin_url();
        $config['plugin_urljs'] = '?wc-api=wc_getjs_mipa_ccard&sn=';
        $config['plugin_urlcss'] = '?wc-api=wc_getcss_mipa_ccard&sn=';
        $config['isAbo'] = $mipa_is_abo;

        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->config: ' . print_r($config, 1));

        // reuse card
        if ($woocommerce->session->mipa_sepa_token == 'reuse') {
            $customerID = $order->get_customer_id();
            $tokens = $this->get_tokens($customerID);
            $payData = '';
            if (array_key_exists($woocommerce->session->mipa_sepa_tokenID, $tokens)) {
                $payData = unserialize($tokens[$woocommerce->session->mipa_sepa_tokenID]->get_token());
            }
            if (empty($payData)) {
                $payDataDefault = unserialize(current($tokens)->get_token());
                if (!empty($payDataDefault)) $payData = $payDataDefault; // fallback to available saved token
            }
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->re-using payData: ' . print_r($payData, 1) . ' Customer: ' . $customerID);
        }
        // if nothing found yet, check POST data (maybe first use of card)
        if (empty($payData) && !empty($_POST['mcpay_sepa_iban'])) {
            $payData = array(
                'holder' => $_POST['mcpay_sepa_holder'],
                'iban' => $_POST['mcpay_sepa_iban'],
                'bic' => $_POST['mcpay_sepa_bic'],
            );
        }

        if (!empty($payData)) {
            // first use of a card
            try {
                $userData = $this->getUserDataFromOrder($order, $config);
                $userData['chksum']['pm'] = 'sepa';

                $res = $this->payMiPa->bookSepaShort($userData, $config, $payData);
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->bookSepa->res: ' . print_r($res,1));
            } catch (Exception $e) {
                $errorMsg = __($e->getMessage(), 'mipa'); // try to translate error
                $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
                wc_add_notice($errorMsg, 'error');
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->bookSepa->error: ' . $errorMsg);
                return FALSE;
            }

            if ($res->resApprove->status == 'APPROVED') {
                if (!empty($config['testMode'])) $order->add_order_note(__('!! TEST MODE !!', 'mipa'));
                $order->add_order_note(__('PAYTOKEN:', 'mipa') . $res->resCreate->payToken);
                $order->add_order_note(__('SESSIONID:', 'mipa') . $res->resCreate->sessionId);

                // save important parameters
                $params = new stdClass();
                $params->sessionId = $res->resCreate->sessionId;
                $params->transactionId = $res->resCreate->payToken;
                $params->mandateTextDE = $res->mandateTextDE;
                $params->mandateTextEN = $res->mandateTextEN;
                $this->saveOrderParams($order_id, $params); // save info to order

                $order->set_transaction_id($res->resCreate->payToken);
                $lang = strtolower($order->get_billing_country());
                if ($lang == 'de') {
                    $order->add_order_note($res->mandateTextDE);
                } else {
                    $order->add_order_note($res->mandateTextEN);
                }
                $order->save();

                // trx successful
                try {
                    $order->payment_complete();
                    if (!$this->isOrderStatus($order->get_status(), $config['status_paid'])) {
                        $order->set_status($config['status_paid']);
                        $order->save();
                    }
                } catch (Exception $e){
                    if ($this->debug) $this->log->debug($e->getMessage(), $e->getCode());

                    $errorMsg = $e->getMessage().' ('.$e->getCode().')';
                    $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
                    wc_add_notice($errorMsg, 'error');

                    // jump to error page
                    if ($this->error_page == 'CART') {
                        $url = $order->get_cancel_order_url(); // cancel order url
                    } else {
                        $url = wc_get_checkout_url(); // checkout url
                    }
                    return array(
                        'result' => 'error',
                        'redirect' => $url
                    );
                }

                $url = $this->get_return_url($order);

                // Save Card for reuse
                if (!empty($_POST[$this->payMiPa->prefixSepa . 'remember'])) {
                    $woocommerce->session->mipa_card_remember_card = TRUE;
                }

                if ($woocommerce->session->mipa_card_remember_card) {
                    $customerID = $order->get_customer_id();
                    $woocommerce->session->mipa_card_remember_card = FALSE;
                    if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->saved payData after success trx: ' . print_r($payData, 1) . ' Customer: ' . $customerID);
                    $this->savePaymentToken($payData, $customerID);
                }

                $return = array(
                    'result' => 'success',
                    'redirect' => $url
                );
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->return: ' . print_r($return,1));
                return $return;
            } else {
                $errorMsg = $res->resCreate->status;
                $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
                wc_add_notice($errorMsg, 'error');

                // jump to error page
                if ($this->error_page == 'CART') {
                    $url = $order->get_cancel_order_url(); // cancel order url
                } else {
                    $url = wc_get_checkout_url(); // checkout url
                }
                return array(
                    'result' => 'error',
                    'redirect' => $url
                );
            }
        }
        return FALSE;
    }

    /**
     * responseAction
     *
     * @throws \Exception
     */
    function responseAction()
    {
        global $woocommerce;

        $this->init_form_fields();
        $this->init_settings();

        // debug info
        if (!empty($_GET['function']) && $_GET['function'] == 'custinfo') {
            $config = $this->settings;
            if ($_GET['ak'] == $config['accessKey']) {

                if (!empty($_GET['logs'])) {
                    $logPath = __DIR__ . '/../../uploads/wc-logs/';
                    $logFiles = $this->payMiPa->findLogFile($logPath);
                    //echo '<pre>'.print_r($logFiles,1).'</pre>';
                    foreach ($logFiles AS $logFile) {
                        $p = explode('-', $logFile);
                        if ($p[2] == date('Y')) {
                            $ld = $p[4] . '-' . $p[3] . '-' . $p[2];
                            $lt = $p[0] . '-' . $p[1];
                            $lr = 1;
                        } else {
                            $ld = $p[3] . '-' . $p[2] . '-' . $p[1];
                            $lt = $p[0];
                            $lr = 1;
                            if ($p[0] == 'log') $lr = 0;
                        }
                        echo '<a href="?wc-api=wc_gateway_mipa_sepa&function=custinfo&ak=' . $config['accessKey'] . '&lt=' . $lt . '&log=' . $ld . '&lr=' . $lr . '" target="_blank">' . $logFile . '</a><br>';
                    }

                } else if (!empty($_GET['log'])) {
                    $day = date('Y-m-d', strtotime($_GET['log']));
                    $logFile = 'log-' . $day;
                    if (!empty($_GET['lt'])) {
                        $logFile = $_GET['lt'] . '-' . $day;
                    }
                    $logPath = __DIR__ . '/../../uploads/wc-logs/';
                    $logFile = $this->payMiPa->findLogFile($logPath, $logFile);
                    echo $logFile . '<br><hr>';
                    $log = $this->payMiPa->readLogFile($logFile, (bool)$_GET['lr'], $day);
                    echo $log;

                } else {

                    // plugin info
                    include_once(dirname(__FILE__) . '/../../../wp-admin/includes/plugin.php');
                    $pInfo = get_plugin_data(dirname(__FILE__) . '/gateway-mipa.php');
                    echo '<pre>' . print_r($pInfo, 1) . '</pre>';
                    // plugin configs
                    $configAll = array();
                    foreach ($this->payMiPa->mcpayEventURLs as $k => $v) {
                        $configAll[$k] = get_option($this->plugin_id . 'mipa_' . $k . '_settings');
                    }
                    echo '<pre>' . print_r($configAll, 1) . '</pre>';

                }
            } else {
                echo 'Not found!';
            }
            exit();
        }

        $crlf = "\n";
        $return = 'status=ok' . $crlf;

        $act = '';
        if (!empty($_POST['action'])) $act = $_POST['action'];
        $type = '';
        if (!empty($_POST['type'])) $type = $_POST['type'];

        if ($act == 'transactionCreate') {
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->GET: ' . print_r($_GET, 1) . ' POST: ' . print_r($_POST, 1));

            $config = $this->settings;
            $sessID = $_POST['sessionId'];
            $trxID = $_POST['transactionId'];
            $amount = $_POST['amount'];
            // check if session id contains underliner
            if (strpos($sessID, '_') === FALSE){
                $orderId = $sessID;
            } else {
                $p = explode('_', $sessID);
                $orderId = $p[1];
            }
            // special case old plugin
            if (empty($orderId)) {
                $p = explode(': ', $_POST['description']);
                $orderId = $p[1];
            }

            switch ($type) {

                case 'BOOKING': // normal trx
                    break;

                case 'REFUND':
                case 'STOP':
                    // detect refund source in case it is from control center it should be booked otherwise not
                    if (!empty($_POST['description']) && substr($_POST['description'], 0, 2) == '##') {
                        // refund from inside woo so do nothing
                    } else {
                        $args = array(
                            'amount' => abs($amount / 100),
                            'reason' => $_POST['description'],
                            'order_id' => $orderId
                        );
                        $res = wc_create_refund($args);
                        if (is_a($res, 'WP_Error')) {
                            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->wc_create_refund (' . $orderId . '): ' . print_r($res, 1));
                            return $res;
                        } else {
                            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->wc_create_refund: ' . print_r($args, 1));
                        }
                    }
                    break;

                case 'REVERSAL': // chargeback
                    $order = new WC_Order($orderId);
                    $order->add_order_note(__('REVERSAL:', 'mipa') . sprintf('%1.2f', ($amount / 100)) . ' ' . $order->get_currency());
                    // change status to wc-pending if not wc-completed
                    if ($order->get_status() != 'completed') {
                        $order->set_status('pending');
                        $order->save();
                    }

                    if (class_exists('WC_Subscriptions_Order')) {
                        $isAbo = FALSE;
                        $isRenewal = FALSE;
                        if (wcs_order_contains_subscription($order)) {
                            //if (WC_Subscriptions_Order::order_contains_subscription($order)) {
                            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->$orderId order_contains_subscription: ' . print_r($orderId, 1));
                            $isAbo = TRUE;
                        }
                        if (WC_Subscriptions_Renewal_Order::is_renewal($order)) {
                            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->$orderId is_renewal: ' . print_r($orderId, 1));
                            $isAbo = TRUE;
                            $isRenewal = TRUE;
                        }

                        if ($isAbo) {
                            if ($isRenewal) {
                                $subIDs = wcs_get_subscriptions_for_renewal_order($orderId);
                            } else {
                                $subIDs = wcs_get_subscriptions_for_order($orderId, array('subscription_status' => 'active'));
                            }

                            $orders = array();
                            foreach ($subIDs AS $k => $v) {
                                $sub = new WC_Subscription($k);
                                $tmp = $sub->get_related_orders();
                                $orders = array_merge($orders, $tmp);
                            }

                            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->WC_Subscriptions_Order::get_users_subscription_orders deactivating subscriptions: ' . print_r($orders, 1));
                            foreach ($orders as $orderID) {
                                WC_Subscriptions_Manager::put_subscription_on_hold_for_order($orderID);
                            }
                        }
                    }
                    break;

                case 'BACKPAY': // backpay the chargeback
                    $order = new WC_Order($orderId);
                    $comment = __('BACKPAY:', 'mipa') . sprintf('%1.2f', ($amount / 100)) . ' ' . $order->get_currency();
                    // change status to wc-processing if wc-pending

                    // check if paid complete
                    $completePaid = FALSE;
                    try {
                        $this->payMiPa->setConfig($config);
                        $this->payMiPa->switchPaymethod('sepa');
                        $sessionInfo = $this->payMiPa->sessionGet($sessID);
                        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->sessionInfo: ' . print_r($sessionInfo, 1));
                        if ($sessionInfo->openAmount <= 0) {
                            $completePaid = TRUE;
                        } else {
                            $comment .= ' ' . __('OPEN_AMOUNT:', 'mipa') . sprintf('%1.2f', ($sessionInfo->openAmount / 100)) . ' ' . $order->get_currency();
                        }
                    } catch (Exception $e) {
                        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->sessionInfo->error: ' . $e->getMessage());
                        return new WP_Error(time(), $e->getMessage());
                    }
                    // special case old plugin
                    if (empty($orderId)) {
                        $orderId = $sessionInfo->freeParams['order_id'];
                        $order = new WC_Order($orderId);
                    }

                    if (!empty($comment)) {
                        $order->add_order_note($comment);
                    }

                    if ($completePaid) {
                        if ($order->get_status() == 'pending') {
                            $order->set_status('processing');
                            $order->save();
                        }
                    }

                    if (class_exists('WC_Subscriptions_Order')) {
                        $isAbo = FALSE;
                        $isRenewal = FALSE;
                        if (wcs_order_contains_subscription($order)) {
                            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->$orderId order_contains_subscription: ' . print_r($orderId, 1));
                            $isAbo = TRUE;
                        }
                        if (WC_Subscriptions_Renewal_Order::is_renewal($order)) {
                            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->$orderId is_renewal: ' . print_r($orderId, 1));
                            $isAbo = TRUE;
                            $isRenewal = TRUE;
                        }

                        if ($isAbo && $completePaid) {
                            if ($isRenewal) {
                                $subIDs = wcs_get_subscriptions_for_renewal_order($orderId);
                            } else {
                                $subIDs = wcs_get_subscriptions_for_order($orderId, array('subscription_status' => 'active'));
                            }

                            $orders = array();
                            foreach ($subIDs AS $k => $v) {
                                $sub = new WC_Subscription($k);
                                $tmp = $sub->get_related_orders();
                                $orders = array_merge($orders, $tmp);
                            }

                            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->WC_Subscriptions_Order::get_users_subscription_orders reactivating subscriptions: ' . print_r($orders, 1));
                            foreach ($orders as $orderID) {
                                WC_Subscriptions_Manager::activate_subscriptions_for_order($orderID);
                            }
                        }
                    }
                    break;
            }

            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->return: ' . print_r($return, 1));
        }
        echo $return;
        exit();
    }

    /**
     * process_refund
     *
     * @param int $order_id
     * @param null $amount
     * @param string $reason
     *
     * @return bool|\WP_Error
     */
    public function process_refund($order_id, $amount = null, $reason = '')
    {
        $order = new WC_Order($order_id);

        $config = $this->settings;
        $config['amount'] = $amount;
        $config['sessid'] = $this->getSessID($order_id);
        $config['payText'] = '##' . $reason; // use ## to identify its coming from woo and not from controlcenter

        if (empty($config['amount']) || $config['amount'] <= 0) {
            return new WP_Error(time(), __('Amount cant be empty!', 'mipa'));
        }
        if (empty($config['sessid'])) {
            return new WP_Error(time(), __('SessionID cant be empty!', 'mipa'));
        }

        try {
            $refundInfo = $this->payMiPa->refundSepa($order, $config);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->refundTRX: ' . print_r($refundInfo, 1));
            $this->saveOrderParams($order_id, $refundInfo, '_mipa_refund_params'); // save refund info to order
            if ($refundInfo->error == 0) {
                return TRUE;
            }
        } catch (Exception $e) {
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->refundTRX->error: ' . $e->getMessage());
            return new WP_Error(time(), $e->getMessage());
        }

        return new WP_Error(time(), __('Unknown Refund Error.', 'mipa'));
    }

    /**
     * savePaymentToken
     *
     * @param $payData
     * @param $customerID
     *
     * @throws \Exception
     */
    function savePaymentToken($payData, $customerID)
    {
        // remove old token
        $tokens = $this->get_tokens($customerID);
        foreach ($tokens AS $k => $token) {
            $payToken = new WC_Payment_Token_ECheck($token->get_id());
            $payToken->delete();
        }
        // save new token
        try {
            $payToken = new WC_Payment_Token_ECheck();
            $payToken->set_token(serialize($payData));
            $payToken->set_gateway_id($this->id);
            $payToken->set_user_id($customerID);
            $payToken->set_last4($payData['iban']);
            $payToken->save();
            WC_Payment_Tokens::set_users_default($customerID, $payToken->get_id());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addContentToEMail
     *
     * @param $order
     * @param $sent_to_admin
     * @param $plain_text
     * @param $email
     */
    function addContentToEMail($order, $sent_to_admin, $plain_text, $email)
    {
        global $woocommerce;

        $emailsToAddText = array(
            //'new_renewal_order', // admin renewal mail
            'customer_processing_renewal_order', // customer renewal mail
            //'new_order', // admin email
            'customer_processing_order', // customer email
        );
        if (in_array($email->id, $emailsToAddText)) {
            $order_id = $order->get_id();
            $lang = $this->getLanguage();
            $params = $this->loadOrderParams($order_id);
            if ($lang == 'de') {
                echo '<p>' . $params->mandateTextDE . '</p>';
            } else {
                echo '<p>' . $params->mandateTextEN . '</p>';
            }
        }
    }

    /**
     * addContentThankyou
     *
     * @param $order_id
     */
    function addContentThankyou($order_id)
    {
        $lang = $this->getLanguage();
        $params = $this->loadOrderParams($order_id);
        if ($lang == 'de') {
            echo '<p>' . $params->mandateTextDE . '</p>';
        } else {
            echo '<p>' . $params->mandateTextEN . '</p>';
        }
    }

}

?>