<?php

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class WC_Gateway_Mipa_PP extends WC_Gateway_Mipa_Base
{

    const id = 'mipa_prepay';

    /**
     * Constructor for the gateway.
     * @access public
     * @return void
     */
    public function __construct()
    {
        $this->id = self::id;
        $this->method_title = __('Micropayment PrePay', 'mipa');
        $this->has_fields = TRUE;

        // Load the settings
        $this->init_form_fields();
        $this->init_settings();

        // Actions
        add_action('woocommerce_api_wc_gateway_' . $this->id, array($this, 'responseAction'));
        add_action('woocommerce_email_after_order_table', array($this, 'addContentToEMail'), 20, 4);
        add_action('woocommerce_thankyou_' . $this->id, array($this, 'addContentThankyou'));

        parent::__construct();
    }

    /**
     * Initialise Gateway Settings Form Fields
     * @access public
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable/Disable', 'mipa'),
                'type' => 'checkbox',
                'label' => __('Enable Plugin', 'mipa'),
                'default' => 'yes'
            ),
            'title_de' => array(
                'title' => __('Title', 'mipa').' DE',
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'mipa'),
                'default' => 'Micropayment Vorkasse',
                'desc_tip' => TRUE,
            ),
            'title_en' => array(
                'title' => __('Title', 'mipa').' EN',
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'mipa'),
                'default' => 'Micropayment PrePay',
                'desc_tip' => TRUE,
            ),
            'description_de' => array(
                'title' => __('Description', 'mipa').' DE',
                'type' => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'mipa'),
                'default' => $this->get_poweredby() . '<b>Bezahlung per Vorkasse</b><br>Bitte überweisen Sie den Rechnungsbetrag per Vorkasse. Die Bankverbindung teilen wir Ihnen in der Auftragsbestätigung mit, die Sie umgehend nach Eingang der Bestellung erhalten. Der Zahlungseingang wird Ihnen von Micropayment GmbH per E-Mail bestätigt. Der Versand der Ware erfolgt nach Zahlungseingang.'
            ),
            'description_en' => array(
                'title' => __('Description', 'mipa').' EN',
                'type' => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'mipa'),
                'default' => $this->get_poweredby() . '<b>Payment with  Bank Transfer</b><br>Please transfer the invoice amount in advance. We will inform you about the bank details with your order confirmation, which you will receive immediately after receipt of the order. The receipt of payment will be confirmed by Micropayment GmbH by e-mail. The product will be shipped once the money has been received.'
            ),

            // BIC: INGDDEFFXXX IBAN: DE12500105170648489890
            'mini_amount' => array(
                'title' => __('Min Amount', 'mipa'),
                'description' => __('Setup the minimum amount for this paymethod. In Euro CENT', 'mipa'),
                'type' => 'text',
                'default' => '1',
                'desc_tip' => TRUE,
            ),
            'maxi_amount' => array(
                'title' => __('Max Amount', 'mipa'),
                'description' => __('Setup the maximum amount for this paymethod. 0 means no Limit! In Euro CENT. In case you need a higher limit, please contact us.', 'mipa'),
                'type' => 'text',
                'default' => '0',
                'desc_tip' => TRUE,
            ),
            'freetext' => array(
                'title' => __('Freetext on settlement', 'mipa'),
                'description' => __('Here you can define a freetext for the customer settlement.', 'mipa').' '.__('Placeholder {$orderid} = OrderID', 'mipa'),
                'type' => 'text',
                'placeholder' => __('Order {$orderid} Your Freetext!!', 'mipa'),
                'default' => '',
                'desc_tip' => __('If empty only order id will be used.', 'mipa'),
            ),
            'expiredays' => array(
                'title' => __('Expire in days', 'mipa'),
                'description' => __('In how many days the prepayment should be paid / reminded.', 'mipa'),
                'type' => 'text',
                'default' => '21',
                'desc_tip' => TRUE,
            ),
            'prepay_text_de' => array(
                'title' => __('Prepay Text German', 'mipa'),
                'desc_tip' => __('Setup the Prepay text in german', 'mipa'),
                'type' => 'textarea',
                'default' => __('Sehr geehrte Kundin, sehr geehrter Kunde,<br><br>'
                    . 'vielen Dank für Ihre Bestellung vom {$orderDateDE}. micropayment™ übernimmt für uns die Zahlungsabwicklung.<br><br>'
                    . 'Bitte überweisen Sie {$amount} {$currency} bis zum {$dueDateDE} auf die folgende Kontoverbindung, um den Kauf abzuschließen.<br><br>'
                    . 'Kontoinhaber: {$accountHolder}<br>'
                    . 'IBAN: {$iban}<br>'
                    . 'BIC: {$bic}<br>'
                    . 'Bank: {$bankName}<br>'
                    . 'Gesamtbetrag: {$amount} {$currency}<br>'
                    . 'Verwendungszweck: {$payText}'
                    , 'mipa'),
                'description' => 'Die folgenden Platzhalter sind verfügbar:<br>'
                    . '{$amount} - Buchungsbetrag<br>'
                    . '{$currency} - Buchungswährung<br>'
                    . '{$dueDate} - Datum Zahlungsziel<br>'
                    . '{$orderDate} - Bestell Datum<br>'
                    . '{$iban} - IBAN<br>'
                    . '{$bic} - BIC<br>'
                    . '{$acountHolder} - Kontoinhaber<br>'
                    . '{$bankName} - Bank Name<br>'
                    . '{$payText} - Verwendungszweck<br>',
            ),
            'prepay_text_en' => array(
                'title' => __('Prepay Text English', 'mipa'),
                'desc_tip' => __('Setup the Prepay text in english', 'mipa'),
                'type' => 'textarea',
                'default' => __(' Dear Customer,<br><br>'
                    . 'Thank you for purchase on {$orderDateEN}. Payment processing for us is provided by micropayment™.<br><br>'
                    . 'Please make transfer in the amount of {$amount} {$currency} by {$dueDateEN} in order to complete your purchase. '
                    . 'Payment should be made to the following bank account.<br><br>'
                    . 'Beneficiary: {$accountHolder}<br>'
                    . 'IBAN: {$iban}<br>'
                    . 'BIC: {$bic}<br>'
                    . 'Bank: {$bankName}<br>'
                    . 'Total amount: {$amount} {$currency}<br>'
                    . 'Payment purpose: {$payText}<br>'
                    , 'mipa'),
                'description' => 'The following placeholder are available:<br>'
                    . '{$amount} - Amount to book<br>'
                    . '{$currency} - Currency to book<br>'
                    . '{$dueDate} - Due Date<br>'
                    . '{$orderDate} - Order Date<br>'
                    . '{$iban} - IBAN<br>'
                    . '{$bic} - BIC<br>'
                    . '{$acountHolder} - Account Holder<br>'
                    . '{$bankname} - Bank Name<br>'
                    . '{$payText} - Usage<br>',
            ),
        );
        $baseFields = $this->get_base_form_fields();
        $this->form_fields = array_merge($baseFields, $this->form_fields);
    }

    /**
     * payment_fields
     * @return bool
     */
    function payment_fields()
    {
        global $woocommerce;

        if ($this->description) {
            echo apply_filters('wc_' . $this->id . '_description', $this->description);
        } else {
            $output = __('You will get a prefilled payform via email at the end.', 'mipa');
            echo $output;
        }

        return TRUE;
    }

    /**
     * validate_fields
     * @return bool
     */
    function validate_fields()
    {
        global $woocommerce;
        return TRUE;
    }

    /**
     * process_payment
     * Process the payment and return the result
     * @access public
     *
     * @param int $order_id
     *
     * @return array
     */
    function process_payment($order_id)
    {
        global $woocommerce;
        $order = new WC_Order($order_id);

        $config = $this->settings;
        $config['plugin_path'] = $this->plugin_path();
        $config['plugin_url'] = $this->plugin_url();
        $config['expiredays'] = $this->expiredays;

        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->config: ' . print_r($config, 1));

        // first use of a card
        try {
            $userData = $this->getUserDataFromOrder($order, $config);

            $res = $this->payMiPa->bookPrepay($userData, $config);
        } catch (Exception $e) {
            $errorMsg = __($e->getMessage(), 'mipa'); // try to translate error
            $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
            wc_add_notice(__('ERROR:', 'mipa') . $errorMsg, 'error');
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->bookPrepay->error: ' . $e->getMessage());
            return FALSE;
        }

        if ($res->resCreate->status == 'INIT') {
            if (!empty($config['testMode'])) $order->add_order_note(__('!! TEST MODE !!', 'mipa'));
            $order->add_order_note(__('PAYTOKEN:', 'mipa') . $res->resCreate->payToken);
            $order->add_order_note(__('SESSIONID:', 'mipa') . $res->resCreate->sessionId);

            // save important parameters
            $params = new stdClass();
            $params->sessionId = $res->resCreate->sessionId;
            $params->transactionId = $res->resCreate->payToken;
            $params->prepayTextDE = $res->prepayTextDE;
            $params->prepayTextEN = $res->prepayTextEN;
            $this->saveOrderParams($order_id, $params); // save info to order

            $url = $this->get_return_url($order);
            //$url = $order->get_checkout_payment_url(TRUE); // debugging

            $order->set_transaction_id($res->resCreate->payToken);
            $lang = strtolower($order->get_billing_country());
            if ($lang == 'de') {
                $order->add_order_note($res->prepayTextDE);
            } else {
                $order->add_order_note($res->prepayTextEN);
            }
            $order->save();

            // trx successful
            if (!$this->isOrderStatus($order->get_status(), $config['status_pending'])) {
                $order->set_status($config['status_pending']);
                $order->save();
            }

            return array(
                'result' => 'success',
                'redirect' => $url
            );

        } else {
            $errorMsg = $res->resCreate->status;
            $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
            wc_add_notice($errorMsg, 'error');

            // jump to error page
            if ($this->error_page == 'CART') {
                $url = $order->get_cancel_order_url(); // cancel order url
            } else {
                $url = wc_get_checkout_url(); // checkout url
            }
            return array(
                'result' => 'error',
                'redirect' => $url
            );
        }

        return FALSE;
    }

    /**
     * responseAction
     *
     * @throws \Exception
     */
    function responseAction()
    {
        global $woocommerce;

        $this->init_form_fields();
        $this->init_settings();

        $crlf = "\n";
        $return = 'status=ok' . $crlf;

        $act = '';
        if (!empty($_POST['action'])) $act = $_POST['action'];
        $type = '';
        if (!empty($_POST['type'])) $type = $_POST['type'];

        if ($act == 'transactionCreate') {
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->GET: ' . print_r($_GET, 1) . ' POST: ' . print_r($_POST, 1));

            $config = $this->settings;
            $sessID = $_POST['sessionId'];
            $trxID = $_POST['transactionId'];
            $amount = $_POST['paidAmount'];
            $p = explode('_', $sessID);
            $orderId = $p[1];
            if (empty($orderId)) $orderId = $p[0];

            switch ($type) {

                case 'CREATE': // normal start of prepay trx
                    break;

                case 'REFUND': // only done if OVERPAID
                    $order = new WC_Order($orderId);
                    $order->add_order_note(__('REFUND OVERPAID: ', 'mipa') . sprintf('%1.2f', (abs($amount) / 100)) . ' ' . $order->get_currency());
                    break;

                case 'EXPIRE': // payment expired
                    $order = new WC_Order($orderId);
                    $order->set_status('cancelled'); // todo: configure status in backend
                    $order->save();
                    break;

                case 'PAYIN':
                    $order = new WC_Order($orderId);
                    $comment = __('PAYIN:', 'mipa') . sprintf('%1.2f', ($amount / 100)) . ' ' . $order->get_currency();
                    // change status to wc-processing if wc-pending

                    // check if paid complete
                    $completePaid = FALSE;
                    try {
                        $this->payMiPa->setConfig($config);
                        $this->payMiPa->switchPaymethod('prepay');
                        $sessionInfo = $this->payMiPa->sessionGet($sessID);
                        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->sessionInfo: ' . print_r($sessionInfo, 1));
                        if ($sessionInfo->openAmount <= 0) {
                            $completePaid = TRUE;
                        } else {
                            $comment .= ' ' . __('OPEN_AMOUNT:', 'mipa') . sprintf('%1.2f', ($sessionInfo->openAmount / 100)) . ' ' . $order->get_currency();
                        }
                    } catch (Exception $e) {
                        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->sessionInfo->error: ' . $e->getMessage());
                        return new WP_Error(time(), $e->getMessage());
                    }

                    if (!empty($comment)) {
                        $order->add_order_note($comment);
                    }

                    if ($completePaid) {
                        if (!$this->isOrderStatus($order->get_status(), $config['status_paid'])) {
                            $order->set_status($config['status_paid']);
                            $order->save();
                        }
                    }
                    break;

                case 'CHANGE':
                    $order = new WC_Order($orderId);
                    // check if paid complete
                    $completePaid = FALSE;
                    try {
                        $this->payMiPa->setConfig($config);
                        $this->payMiPa->switchPaymethod('prepay');
                        $sessionInfo = $this->payMiPa->sessionGet($sessID);
                        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->sessionInfo: ' . print_r($sessionInfo, 1));
                        $comment = __('Order Amount changed to:', 'mipa') . sprintf('%1.2f', ($sessionInfo->orderAmount / 100)) . ' ' . $order->get_currency();
                        if ($sessionInfo->openAmount <= 0) {
                            $completePaid = TRUE;
                        } else {
                            $comment .= ' ' . __('OPEN_AMOUNT:', 'mipa') . sprintf('%1.2f', ($sessionInfo->openAmount / 100)) . ' ' . $order->get_currency();
                        }
                    } catch (Exception $e) {
                        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->sessionInfo->error: ' . $e->getMessage());
                        return new WP_Error(time(), $e->getMessage());
                    }

                    if (!empty($comment)) {
                        $order->add_order_note($comment);
                    }

                    if ($completePaid) {
                        if (!$this->isOrderStatus($order->get_status(), $config['status_paid'])) {
                            $order->set_status($config['status_paid']);
                            $order->save();
                        }
                    }
                    break;
            }

            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->return: ' . print_r($return, 1));

        } else if ($act == 'sessionEmail') {
            $sessID = $_POST['sessionId'];
            $p = explode('_', $sessID);
            $orderId = $p[1];
            // activate lines which emails you want to log
            $allowedTypes = array(
                //'CREATE', // Zahlung erzeugt
                //'CHANGE', // Zahlung geändert
                //'PAYIN', // Zahlung eingegangen
                'REMIND', // Erinnerung
                'LASTREMIND', // Letzte Erinnerung
                'EXPIRE', // Abgelaufen
            ); // only allowed values need to be logged
            if ($orderId > 0 && in_array($type, $allowedTypes)) {
                $order = new WC_Order($orderId);
                $order->add_order_note(__('EMAIL SENT: ', 'mipa') . $type . ' ' . date('d.m.Y H:i:s'));
            }
        }
        echo $return;
        exit();
    }

    /**
     * process_refund
     *
     * @param int $order_id
     * @param null $amount
     * @param string $reason
     *
     * @return bool|\WP_Error
     */
    public function process_refund($order_id, $amount = null, $reason = '')
    {
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__, print_r(func_get_args(),1).print_r($_POST,1).print_r($_GET,1));
        $order = new WC_Order($order_id);

        $config = $this->settings;
        $config['amount'] = $amount;
        $params = $this->loadOrderParams($order_id);
        $config['sessid'] = $params->resCreate->sessionId;
        $config['payText'] = '##' . $reason; // use ## to identify its coming from woo and not from controlcenter

        if (empty($config['amount']) || $config['amount'] <= 0) {
            return new WP_Error(time(), __('Amount cant be empty!', 'mipa'));
        }
        if (empty($config['sessid'])) {
            return new WP_Error(time(), __('SessionID cant be empty!', 'mipa'));
        }

        try {
            $refundInfo = $this->payMiPa->refundPrepay($order, $config);
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->refundTRX: ' . print_r($refundInfo, 1));
            $this->saveOrderParams($order_id, $refundInfo, '_mipa_refund_params'); // save refund info to order
            if ($refundInfo->error == 0) {
                return TRUE;
            }
        } catch (Exception $e) {
            if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->refundTRX->error: ' . $e->getMessage());
            return new WP_Error(time(), $e->getMessage());
        }

        return new WP_Error(time(), __('Unknown Refund Error.', 'mipa'));
    }

    /**
     * addContentToEMail
     *
     * @param $order
     * @param $sent_to_admin
     * @param $plain_text
     * @param $email
     */
    function addContentToEMail($order, $sent_to_admin, $plain_text, $email)
    {
        global $woocommerce;

        $emailsToAddText = array(
            //'new_renewal_order', // admin renewal mail
            'customer_processing_renewal_order', // customer renewal mail
            //'new_order', // admin email
            'customer_processing_order', // customer email
            'customer_on_hold_order',
        );
        if (in_array($email->id, $emailsToAddText)) {
            $order_id = $order->get_id();
            $lang = $this->getLanguage();
            $params = $this->loadOrderParams($order_id);
            if ($lang == 'de') {
                echo '<p>' . $params->prepayTextDE . '</p>';
            } else {
                echo '<p>' . $params->prepayTextEN . '</p>';
            }
        }
    }

    /**
     * addContentThankyou
     *
     * @param $order_id
     */
    function addContentThankyou($order_id)
    {
        $lang = $this->getLanguage();
        $params = $this->loadOrderParams($order_id);
        if ($lang == 'de') {
            echo '<p>' . $params->prepayTextDE . '</p>';
        } else {
            echo '<p>' . $params->prepayTextEN . '</p>';
        }
    }

}

?>