<?php

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class WC_Gateway_Mipa_PPW extends WC_Gateway_Mipa_Base
{

    const id = 'mipa_prepay_web';

    /**
     * Constructor for the gateway.
     * @access public
     * @return void
     */
    public function __construct()
    {
        $this->id = self::id;
        $this->method_title = __('Micropayment PrePay', 'mipa').' Web';
        $this->has_fields = TRUE;

        // Load the settings
        $this->init_form_fields();
        $this->init_settings();

        // Actions
        add_action('woocommerce_api_wc_gateway_' . $this->id, array($this, 'responseBaseAction'));

        parent::__construct();

        $this->supports = array('products');
    }

    /**
     * Initialise Gateway Settings Form Fields
     * @access public
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable/Disable', 'mipa'),
                'type' => 'checkbox',
                'label' => __('Enable Plugin', 'mipa'),
                'default' => 'yes'
            ),
            'title_de' => array(
                'title' => __('Title', 'mipa').' DE',
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'mipa'),
                'default' => 'Micropayment Vorkasse',
                'desc_tip' => TRUE,
            ),
            'title_en' => array(
                'title' => __('Title', 'mipa').' EN',
                'type' => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'mipa'),
                'default' => 'Micropayment PrePay',
                'desc_tip' => TRUE,
            ),
            'description_de' => array(
                'title' => __('Description', 'mipa').' DE',
                'type' => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'mipa'),
                'default' => $this->get_poweredby() . '<b>Bezahlung per Vorkasse</b><br>Bitte überweisen Sie den Rechnungsbetrag per Vorkasse. Die Bankverbindung teilen wir Ihnen in der Auftragsbestätigung mit, die Sie umgehend nach Eingang der Bestellung erhalten. Der Zahlungseingang wird Ihnen von Micropayment GmbH per E-Mail bestätigt. Der Versand der Ware erfolgt nach Zahlungseingang.'
            ),
            'description_en' => array(
                'title' => __('Description', 'mipa').' EN',
                'type' => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'mipa'),
                'default' => $this->get_poweredby() . '<b>Payment with  Bank Transfer</b><br>Please transfer the invoice amount in advance. We will inform you about the bank details with your order confirmation, which you will receive immediately after receipt of the order. The receipt of payment will be confirmed by Micropayment GmbH by e-mail. The product will be shipped once the money has been received.'
            ),

            'mini_amount' => array(
                'title' => __('Min Amount', 'mipa'),
                'description' => __('Setup the minimum amount for this paymethod. In Euro CENT', 'mipa'),
                'type' => 'text',
                'default' => '1',
                'desc_tip' => TRUE,
            ),
            'maxi_amount' => array(
                'title' => __('Max Amount', 'mipa'),
                'description' => __('Setup the maximum amount for this paymethod. 0 means no Limit! In Euro CENT. In case you need a higher limit, please contact us.', 'mipa'),
                'type' => 'text',
                'default' => '200000',
                'desc_tip' => TRUE,
            ),
            'freetext' => array(
                'title' => __('Freetext on settlement', 'mipa'),
                'description' => __('Here you can define a freetext for the customer settlement.', 'mipa').' '.__('Placeholder {$orderid} = OrderID', 'mipa'),
                'type' => 'text',
                'placeholder' => __('Order {$orderid} Your Freetext!!', 'mipa'),
                'default' => '',
                'desc_tip' => __('If empty only order id will be used.', 'mipa'),
            ),

            'theme' => array(
                'title' => __('Theme', 'mipa'),
                'description' => __('Choose Theme x1 or x2', 'mipa'),
                'type' => 'text',
                'default' => 'x1',
                'desc_tip' => TRUE,
            ),
            'gfx' => array(
                'title' => __('Logo in pay window', 'mipa'),
                'description' => __('Choose Logo for pay window. Only in x1 theme', 'mipa'),
                'type' => 'text',
                'default' => '',
                'desc_tip' => TRUE,
            ),
            'bgcolor' => array(
                'title' => __('Background Color', 'mipa'),
                'description' => __('Choose Background Color', 'mipa'),
                'type' => 'text',
                'default' => '#ffffff',
                'desc_tip' => TRUE,
            ),
            'bggfx' => array(
                'title' => __('Background Picture', 'mipa'),
                'description' => __('Choose Background Picture', 'mipa'),
                'type' => 'text',
                'default' => '',
                'desc_tip' => TRUE,
            ),
            'suffix' => array(
                'title' => __('Micropayment Domain', 'mipa'),
                'description' => __('Choose the Micropayment Domain on which you are registered.', 'mipa'),
                'type' => 'select',
                'default' => 'micropayment.de',
                'desc_tip' => TRUE,
                'options' => array(
                    'micropayment.de' => 'Micropayment GmbH (.de)',
                    'micropayment.ch' => 'Micropayment AG (.ch)'
                )
            ),
            'secretfield' => array(
                'title' => __('Secretfield', 'mipa'),
                'description' => __('Secretfield for communication validation. No check if empty.', 'mipa'),
                'type' => 'text',
                'default' => '',
                'desc_tip' => TRUE,
            ),

        );
        $baseFields = $this->get_base_form_fields();
        unset($baseFields['status_pending']); // prepay only
        $this->form_fields = array_merge($baseFields, $this->form_fields);
    }

    /**
     * payment_fields
     * @return bool
     */
    function payment_fields()
    {
        global $woocommerce;

        if ($this->description) {
            echo apply_filters('wc_' . $this->id . '_description', $this->description);
        } else {
            $output = __('Pay safe and quick with Prepayment', 'mipa');
            echo $output;
        }

        return TRUE;
    }

    /**
     * validate_fields
     * @return bool
     */
    function validate_fields()
    {
        return TRUE;
    }

    /**
     * process_payment
     * Process the payment and return the result
     * @access public
     *
     * @param int $order_id
     *
     * @return array
     */
    function process_payment($order_id)
    {
        global $woocommerce;
        $order = new WC_Order($order_id);

        $config = $this->settings;
        $config['plugin_path'] = $this->plugin_path();
        $config['plugin_url'] = $this->plugin_url();

        $userData = $this->getUserDataFromOrder($order, $config);

        $url = $this->payMiPa->getPaymentWindowParams($userData, $config, 'prepay');

        if (empty($url)) {
            $errorMsg = __('Cant generate Payment URL.', 'mipa');
            $order->add_order_note(__('ERROR:', 'mipa') . $errorMsg);
            wc_add_notice($errorMsg, 'error');

            // jump to error page
            if ($this->error_page == 'CART') {
                $url = $order->get_cancel_order_url(); // cancel order url
            } else {
                $url = wc_get_checkout_url(); // checkout url
            }
            return array(
                'result' => 'error',
                'redirect' => $url
            );
        } else {
            return array(
                'result' => 'success',
                'redirect' => $url
            );
        }
        return FALSE;
    }

    /**
     * responseAction
     *
     * @param $func
     * @param $order
     *
     * @return string
     */
    function responseAction($func, $order)
    {
        $crlf = "\n";
        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->func (' . $func . '): ' . $order->get_id());
        $return = 'status=ok' . $crlf;

        $amount = 0;
        if (!empty($_GET['amount'])){
            $amount = $_GET['amount'];
        }
        $config = $this->settings;

        switch ($func) {
            case 'init': // normal start of prepay trx
                $order->add_order_note(__('PREPAY INIT AUTH:', 'mipa') . $_GET['auth']);
                $return = 'status=ok' . $crlf // ok  | error
                    . 'url=' . $this->get_return_url($order) . $crlf
                    . 'target=_top' . $crlf
                    . 'forward=1' . $crlf
                    . 'message=OK' . $crlf;
                break;

            case 'expire': // payment expired
                $order->set_status('cancelled'); // todo: configure status in backend
                $order->save();
                break;

            case 'payin':
                $comment = __('PAYIN:', 'mipa') . sprintf('%1.2f', ($amount / 100)) . ' ' . $order->get_currency();
                $comment.= 'OPEN: '.($_GET['openamount']/100).' PAID: '.($_GET['paidamount']/100);
                // change status to wc-processing if wc-pending

                // check if paid complete
                $completePaid = FALSE;
                if ($_GET['openamount'] <= 0){
                    $completePaid = TRUE;
                }

                if (!empty($comment)) {
                    $order->add_order_note($comment);
                }

                if ($completePaid) {
                    if (!$this->isOrderStatus($order->get_status(), $config['status_paid'])) {
                        $order->set_status($config['status_paid']);
                        $order->save();
                    }
                }
                break;

            case 'change':
                $comment = __('CHANGE:', 'mipa') . sprintf('%1.2f', ($amount / 100)) . ' ' . $order->get_currency();
                $comment.= 'OPEN: '.$_GET['openamount'].' PAID: '.$_GET['paidamount'];

                // check if paid complete
                $completePaid = FALSE;
                if ($_GET['openamount'] <= 0){
                    $completePaid = TRUE;
                }

                if (!empty($comment)) {
                    $order->add_order_note($comment);
                }

                if ($completePaid) {
                    if (!$this->isOrderStatus($order->get_status(), $config['status_paid'])) {
                        $order->set_status($config['status_paid']);
                        $order->save();
                    }
                }
                break;

            default:
                if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->missing function: ' . $func);
                $return = 'status=error' . $crlf . 'message=Missing function.' . $crlf;
                break;
        }
        if ($this->debug) $this->log->debug(__CLASS__ . '->' . __FUNCTION__ . '->return: ' . print_r($return, 1));
        return $return;
    }

}

?>