<?php
/*
Plugin Name: WooCommerce Micropayment Gateway
Plugin URI: https://www.micropayment.de/help/integration_shopmodule/detail/?detail=woocommerce
Description: Micropayment gateway for woocommerce
Author: Micropayment (TE)
Author URI: https://r91.micropayment.de/
Contributors:
Donate link: https://r91.micropayment.de/
Tags: payment, gateway, ecommerce, e-commerce, store, sales, sell, shop, checkout, paypal, woo commerce, creditcard, sepa, sofort, paysafecard
Text Domain: mipa
Domain Path: /languages

Version: 1.9.8
Requires at least: 4.7
Tested up to: 5.8.2
Requires PHP: 5.2.4
Stable tag: 1.9.0

WC requires at least: 3.0
WC tested up to: 5.9.0
WCS tested up to: 3.1.0

License:     GPL2
License URI: https://www.gnu.org/licenses/gpl-2.0.html
*/

add_action('plugins_loaded', 'woocommerce_mipa_init', 0);

/**
 * Initialize the gateway.
 *
 * @since 1.0.0
 */
function woocommerce_mipa_init()
{

    if (!class_exists('WC_Payment_Gateway')) return;

    load_plugin_textdomain('mipa', false, dirname(plugin_basename(__FILE__)) . '/languages/'); // load mo files for translation
    load_plugin_textdomain('mipa_error', false, dirname(plugin_basename(__FILE__)) . '/languages/'); // load mo files for translation

    require_once(plugin_basename('class-wc-gateway-mipa_base.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_cc.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_dd.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_pp.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_ccw.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_ddw.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_ppw.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_su.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_pa.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_ps.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_hp.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_cp.php'));
    require_once(plugin_basename('class-wc-gateway-mipa_pd.php'));

    add_filter('woocommerce_payment_gateways', 'woocommerce_mipa_add_gateway');

} // End woocommerce_mipa_init()

/**
 * Add the gateway to WooCommerce
 *
 * @since 1.0.0
 */
function woocommerce_mipa_add_gateway($methods)
{
    $methods[] = 'WC_Gateway_Mipa_CC';
    $methods[] = 'WC_Gateway_Mipa_DD';
    $methods[] = 'WC_Gateway_Mipa_PP';
    $methods[] = 'WC_Gateway_Mipa_CCW';
    $methods[] = 'WC_Gateway_Mipa_DDW';
    $methods[] = 'WC_Gateway_Mipa_PPW';
    $methods[] = 'WC_Gateway_Mipa_SU';
    $methods[] = 'WC_Gateway_Mipa_PA';
    $methods[] = 'WC_Gateway_Mipa_PS';
    $methods[] = 'WC_Gateway_Mipa_HP';
    $methods[] = 'WC_Gateway_Mipa_CP';
    $methods[] = 'WC_Gateway_Mipa_PD';
    return $methods;
} // End woocommerce_mipa_add_gateway()

/**
 * Include links on the Plugins administration screen
 *
 * @param mixed $links
 */
function mcpay_action_links($links)
{
    $pInfo = get_plugin_data(__FILE__);
    $plugin_links = array(
        '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout') . '">Einstellungen</a>',
        //'<a href="mailto:support@micropayment.de?subject=Hilfe zu '..'&body='.$info.'" target="_top">Support</a>',
        '<a href="mailto:support@micropayment.de?subject=Hilfe zu ' . $pInfo['Name'] . ' Version: ' . $pInfo['Version'] . ' installiert auf ' . $_SERVER['HTTP_HOST'] . '" target="_top">Support</a>',
    );

    return array_merge($plugin_links, $links);
}

add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'mcpay_action_links');